/* Generated by Cython 0.25.2 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "depends": []
    },
    "module_name": "yarl._quoting"
}
END: Cython Metadata */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03020000)
    #error Cython requires Python 2.6+ or Python 3.2+.
#else
#define CYTHON_ABI "0_25_2"
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x03030000 || (PY_MAJOR_VERSION == 2 && PY_VERSION_HEX >= 0x02070000)
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef METH_FASTCALL
  #define METH_FASTCALL 0x80
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject **args,
                                              Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : PyInstanceMethod_New(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif


#define __PYX_ERR(f_index, lineno, Ln_error) \
{ \
  __pyx_filename = __pyx_f[f_index]; __pyx_lineno = lineno; __pyx_clineno = __LINE__; goto Ln_error; \
}

#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#endif

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__yarl___quoting
#define __PYX_HAVE_API__yarl___quoting
#include <stdint.h>
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#ifdef PYREX_WITHOUT_ASSERTIONS
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER) && defined (_M_X64)
    #define __Pyx_sst_abs(value) _abs64(value)
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyObject_AsSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
#if PY_MAJOR_VERSION < 3
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u)
{
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#else
#define __Pyx_Py_UNICODE_strlen Py_UNICODE_strlen
#endif
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
#define __Pyx_PyBool_FromLong(b) ((b) ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False))
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c));
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */

static PyObject *__pyx_m;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "yarl/_quoting.pyx",
};

/*--- Type declarations ---*/
struct __pyx_opt_args_4yarl_8_quoting__do_unquote;

/* "yarl/_quoting.pyx":185
 * 
 * 
 * cdef str _do_unquote(str val, str unsafe='', bint qs=False):             # <<<<<<<<<<<<<<
 *     cdef str pct = ''
 *     cdef str last_pct = ''
 */
struct __pyx_opt_args_4yarl_8_quoting__do_unquote {
  int __pyx_n;
  PyObject *unsafe;
  int qs;
};

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* ArgTypeTest.proto */
static CYTHON_INLINE int __Pyx_ArgTypeTest(PyObject *obj, PyTypeObject *type, int none_allowed,
    const char *name, int exact);

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = PyThreadState_GET();
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* GetItemIntUnicode.proto */
#define __Pyx_GetItemInt_Unicode(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Unicode_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "string index out of range"), (Py_UCS4)-1))
static CYTHON_INLINE Py_UCS4 __Pyx_GetItemInt_Unicode_Fast(PyObject* ustring, Py_ssize_t i,
                                                           int wraparound, int boundscheck);

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#endif

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* unicode_iter.proto */
static CYTHON_INLINE int __Pyx_init_unicode_iteration(
    PyObject* ustring, Py_ssize_t *length, void** data, int *kind);

/* PyObjectCallMethod1.proto */
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg);

/* ByteArrayAppend.proto */
static CYTHON_INLINE int __Pyx_PyByteArray_Append(PyObject* bytearray, int value);

/* ByteArrayAppendObject.proto */
static CYTHON_INLINE int __Pyx_PyByteArray_AppendObject(PyObject* bytearray, PyObject* value);

/* PyUnicode_Substring.proto */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_Substring(
            PyObject* text, Py_ssize_t start, Py_ssize_t stop);

/* decode_c_bytes.proto */
static CYTHON_INLINE PyObject* __Pyx_decode_c_bytes(
         const char* cstring, Py_ssize_t length, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors));

/* decode_bytearray.proto */
static CYTHON_INLINE PyObject* __Pyx_decode_bytearray(
         PyObject* string, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors)) {
    return __Pyx_decode_c_bytes(
        PyByteArray_AS_STRING(string), PyByteArray_GET_SIZE(string),
        start, stop, encoding, errors, decode_func);
}

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* PyUnicodeContains.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_ContainsTF(PyObject* substring, PyObject* text, int eq) {
    int result = PyUnicode_Contains(text, substring);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        Py_SIZE(list) = len+1;
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* SliceObject.proto */
#define __Pyx_PyObject_DelSlice(obj, cstart, cstop, py_start, py_stop, py_slice, has_cstart, has_cstop, wraparound)\
    __Pyx_PyObject_SetSlice(obj, (PyObject*)NULL, cstart, cstop, py_start, py_stop, py_slice, has_cstart, has_cstop, wraparound)
static CYTHON_INLINE int __Pyx_PyObject_SetSlice(
        PyObject* obj, PyObject* value, Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** py_start, PyObject** py_stop, PyObject** py_slice,
        int has_cstart, int has_cstop, int wraparound);

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* IncludeStringH.proto */
#include <string.h>

/* BytesEquals.proto */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeAsUCS4.proto */
static CYTHON_INLINE Py_UCS4 __Pyx_PyUnicode_AsPy_UCS4(PyObject*);

/* object_ord.proto */
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyObject_Ord(c)\
    (likely(PyUnicode_Check(c)) ? (long)__Pyx_PyUnicode_AsPy_UCS4(c) : __Pyx__PyObject_Ord(c))
#else
#define __Pyx_PyObject_Ord(c) __Pyx__PyObject_Ord(c)
#endif
static long __Pyx__PyObject_Ord(PyObject* c);

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* SliceObject.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(
        PyObject* obj, Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** py_start, PyObject** py_stop, PyObject** py_slice,
        int has_cstart, int has_cstop, int wraparound);

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* GetModuleGlobalName.proto */
static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name);

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* PyUCS4InUnicode.proto */
static CYTHON_INLINE int __Pyx_UnicodeContainsUCS4(PyObject* unicode, Py_UCS4 character);
static CYTHON_INLINE int __Pyx_PyUnicodeBufferContainsUCS4(Py_UNICODE* buffer, Py_ssize_t length, Py_UCS4 character);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntFromPy.proto */
static CYTHON_INLINE uint8_t __Pyx_PyInt_As_uint8_t(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);


/* Module declarations from 'libc.stdint' */

/* Module declarations from 'yarl._quoting' */
static PyObject *__pyx_v_4yarl_8_quoting_GEN_DELIMS = 0;
static PyObject *__pyx_v_4yarl_8_quoting_SUB_DELIMS_WITHOUT_QS = 0;
static PyObject *__pyx_v_4yarl_8_quoting_SUB_DELIMS = 0;
static PyObject *__pyx_v_4yarl_8_quoting_RESERVED = 0;
static PyObject *__pyx_v_4yarl_8_quoting_UNRESERVED = 0;
static PyObject *__pyx_v_4yarl_8_quoting_ALLOWED = 0;
static CYTHON_INLINE Py_UCS4 __pyx_f_4yarl_8_quoting__hex(uint8_t); /*proto*/
static CYTHON_INLINE int __pyx_f_4yarl_8_quoting__from_hex(Py_UCS4); /*proto*/
static PyObject *__pyx_f_4yarl_8_quoting__do_quote(PyObject *, PyObject *, PyObject *, int, int); /*proto*/
static PyObject *__pyx_f_4yarl_8_quoting__do_unquote(PyObject *, struct __pyx_opt_args_4yarl_8_quoting__do_unquote *__pyx_optional_args); /*proto*/
#define __Pyx_MODULE_NAME "yarl._quoting"
int __pyx_module_is_main_yarl___quoting = 0;

/* Implementation of 'yarl._quoting' */
static PyObject *__pyx_builtin_TypeError;
static PyObject *__pyx_builtin_ValueError;
static PyObject *__pyx_builtin_UnicodeDecodeError;
static PyObject *__pyx_builtin_hex;
static const char __pyx_k_[] = "";
static const char __pyx_k__3[] = "+&=;";
static const char __pyx_k__7[] = "+=&;";
static const char __pyx_k__8[] = "%";
static const char __pyx_k__9[] = "+";
static const char __pyx_k_qs[] = "qs";
static const char __pyx_k__10[] = " ";
static const char __pyx_k__12[] = ":/?#[]@";
static const char __pyx_k__13[] = "!$'()*,";
static const char __pyx_k__14[] = "+?=;";
static const char __pyx_k__15[] = "-._~";
static const char __pyx_k_hex[] = "hex";
static const char __pyx_k_val[] = "val";
static const char __pyx_k_base[] = "base";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_safe[] = "safe";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_quote[] = "_quote";
static const char __pyx_k_upper[] = "upper";
static const char __pyx_k_utf_8[] = "utf-8";
static const char __pyx_k_append[] = "append";
static const char __pyx_k_digits[] = "digits";
static const char __pyx_k_encode[] = "encode";
static const char __pyx_k_errors[] = "errors";
static const char __pyx_k_format[] = "format";
static const char __pyx_k_ignore[] = "ignore";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_strict[] = "strict";
static const char __pyx_k_string[] = "string";
static const char __pyx_k_unsafe[] = "unsafe";
static const char __pyx_k_unquote[] = "_unquote";
static const char __pyx_k_TypeError[] = "TypeError";
static const char __pyx_k_protected[] = "protected";
static const char __pyx_k_ValueError[] = "ValueError";
static const char __pyx_k_Unallowed_PCT[] = "Unallowed PCT %{}{}";
static const char __pyx_k_ascii_letters[] = "ascii_letters";
static const char __pyx_k_yarl__quoting[] = "yarl._quoting";
static const char __pyx_k_Unallowed_PCT_2[] = "Unallowed PCT %{}";
static const char __pyx_k_Unallowed_PCT_3[] = "Unallowed PCT %";
static const char __pyx_k_UnicodeDecodeError[] = "UnicodeDecodeError";
static const char __pyx_k_Argument_should_be_str[] = "Argument should be str";
static const char __pyx_k_home_travis_build_aio_libs_yarl[] = "/home/travis/build/aio-libs/yarl/yarl/_quoting.pyx";
static PyObject *__pyx_kp_u_;
static PyObject *__pyx_kp_u_Argument_should_be_str;
static PyObject *__pyx_n_s_TypeError;
static PyObject *__pyx_kp_u_Unallowed_PCT;
static PyObject *__pyx_kp_u_Unallowed_PCT_2;
static PyObject *__pyx_kp_u_Unallowed_PCT_3;
static PyObject *__pyx_n_s_UnicodeDecodeError;
static PyObject *__pyx_n_s_ValueError;
static PyObject *__pyx_kp_u__10;
static PyObject *__pyx_kp_u__12;
static PyObject *__pyx_kp_u__13;
static PyObject *__pyx_kp_u__14;
static PyObject *__pyx_kp_u__15;
static PyObject *__pyx_kp_u__3;
static PyObject *__pyx_kp_u__7;
static PyObject *__pyx_kp_u__8;
static PyObject *__pyx_kp_u__9;
static PyObject *__pyx_n_s_append;
static PyObject *__pyx_n_s_ascii_letters;
static PyObject *__pyx_n_s_base;
static PyObject *__pyx_n_s_digits;
static PyObject *__pyx_n_s_encode;
static PyObject *__pyx_n_s_errors;
static PyObject *__pyx_n_s_format;
static PyObject *__pyx_n_s_hex;
static PyObject *__pyx_kp_s_home_travis_build_aio_libs_yarl;
static PyObject *__pyx_n_u_ignore;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_protected;
static PyObject *__pyx_n_s_qs;
static PyObject *__pyx_n_s_quote;
static PyObject *__pyx_n_s_safe;
static PyObject *__pyx_n_s_strict;
static PyObject *__pyx_n_u_strict;
static PyObject *__pyx_n_s_string;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_unquote;
static PyObject *__pyx_n_s_unsafe;
static PyObject *__pyx_n_s_upper;
static PyObject *__pyx_kp_u_utf_8;
static PyObject *__pyx_n_s_val;
static PyObject *__pyx_n_s_yarl__quoting;
static PyObject *__pyx_pf_4yarl_8_quoting__quote(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_val, PyObject *__pyx_v_safe, PyObject *__pyx_v_protected, int __pyx_v_qs, int __pyx_v_strict); /* proto */
static PyObject *__pyx_pf_4yarl_8_quoting_2_unquote(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_val, PyObject *__pyx_v_unsafe, PyObject *__pyx_v_qs); /* proto */
static PyObject *__pyx_int_2;
static PyObject *__pyx_int_16;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__5;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_slice__11;
static PyObject *__pyx_tuple__16;
static PyObject *__pyx_tuple__18;
static PyObject *__pyx_codeobj__17;
static PyObject *__pyx_codeobj__19;

/* "yarl/_quoting.pyx":20
 * 
 * 
 * cdef inline Py_UCS4 _hex(uint8_t v):             # <<<<<<<<<<<<<<
 *     if v < 10:
 *         return <Py_UCS4>(v+0x30)  # ord('0') == 0x30
 */

static CYTHON_INLINE Py_UCS4 __pyx_f_4yarl_8_quoting__hex(uint8_t __pyx_v_v) {
  Py_UCS4 __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("_hex", 0);

  /* "yarl/_quoting.pyx":21
 * 
 * cdef inline Py_UCS4 _hex(uint8_t v):
 *     if v < 10:             # <<<<<<<<<<<<<<
 *         return <Py_UCS4>(v+0x30)  # ord('0') == 0x30
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_v < 10) != 0);
  if (__pyx_t_1) {

    /* "yarl/_quoting.pyx":22
 * cdef inline Py_UCS4 _hex(uint8_t v):
 *     if v < 10:
 *         return <Py_UCS4>(v+0x30)  # ord('0') == 0x30             # <<<<<<<<<<<<<<
 *     else:
 *         return <Py_UCS4>(v+0x41-10)  # ord('A') == 0x41
 */
    __pyx_r = ((Py_UCS4)(__pyx_v_v + 0x30));
    goto __pyx_L0;

    /* "yarl/_quoting.pyx":21
 * 
 * cdef inline Py_UCS4 _hex(uint8_t v):
 *     if v < 10:             # <<<<<<<<<<<<<<
 *         return <Py_UCS4>(v+0x30)  # ord('0') == 0x30
 *     else:
 */
  }

  /* "yarl/_quoting.pyx":24
 *         return <Py_UCS4>(v+0x30)  # ord('0') == 0x30
 *     else:
 *         return <Py_UCS4>(v+0x41-10)  # ord('A') == 0x41             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __pyx_r = ((Py_UCS4)((__pyx_v_v + 0x41) - 10));
    goto __pyx_L0;
  }

  /* "yarl/_quoting.pyx":20
 * 
 * 
 * cdef inline Py_UCS4 _hex(uint8_t v):             # <<<<<<<<<<<<<<
 *     if v < 10:
 *         return <Py_UCS4>(v+0x30)  # ord('0') == 0x30
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "yarl/_quoting.pyx":27
 * 
 * 
 * cdef inline int _from_hex(Py_UCS4 v):             # <<<<<<<<<<<<<<
 *     if '0' <= v <= '9':
 *         return <int>(v) - 0x30  # ord('0') == 0x30
 */

static CYTHON_INLINE int __pyx_f_4yarl_8_quoting__from_hex(Py_UCS4 __pyx_v_v) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("_from_hex", 0);

  /* "yarl/_quoting.pyx":28
 * 
 * cdef inline int _from_hex(Py_UCS4 v):
 *     if '0' <= v <= '9':             # <<<<<<<<<<<<<<
 *         return <int>(v) - 0x30  # ord('0') == 0x30
 *     elif 'A' <= v <= 'F':
 */
  __pyx_t_1 = (48 <= __pyx_v_v);
  if (__pyx_t_1) {
    __pyx_t_1 = (__pyx_v_v <= 57);
  }
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "yarl/_quoting.pyx":29
 * cdef inline int _from_hex(Py_UCS4 v):
 *     if '0' <= v <= '9':
 *         return <int>(v) - 0x30  # ord('0') == 0x30             # <<<<<<<<<<<<<<
 *     elif 'A' <= v <= 'F':
 *         return <int>(v) - 0x41 + 10  # ord('A') == 0x41
 */
    __pyx_r = (((int)__pyx_v_v) - 0x30);
    goto __pyx_L0;

    /* "yarl/_quoting.pyx":28
 * 
 * cdef inline int _from_hex(Py_UCS4 v):
 *     if '0' <= v <= '9':             # <<<<<<<<<<<<<<
 *         return <int>(v) - 0x30  # ord('0') == 0x30
 *     elif 'A' <= v <= 'F':
 */
  }

  /* "yarl/_quoting.pyx":30
 *     if '0' <= v <= '9':
 *         return <int>(v) - 0x30  # ord('0') == 0x30
 *     elif 'A' <= v <= 'F':             # <<<<<<<<<<<<<<
 *         return <int>(v) - 0x41 + 10  # ord('A') == 0x41
 *     elif 'a' <= v <= 'f':
 */
  __pyx_t_2 = (65 <= __pyx_v_v);
  if (__pyx_t_2) {
    __pyx_t_2 = (__pyx_v_v <= 70);
  }
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {

    /* "yarl/_quoting.pyx":31
 *         return <int>(v) - 0x30  # ord('0') == 0x30
 *     elif 'A' <= v <= 'F':
 *         return <int>(v) - 0x41 + 10  # ord('A') == 0x41             # <<<<<<<<<<<<<<
 *     elif 'a' <= v <= 'f':
 *         return <int>(v) - 0x61 + 10  # ord('a') == 0x61
 */
    __pyx_r = ((((int)__pyx_v_v) - 0x41) + 10);
    goto __pyx_L0;

    /* "yarl/_quoting.pyx":30
 *     if '0' <= v <= '9':
 *         return <int>(v) - 0x30  # ord('0') == 0x30
 *     elif 'A' <= v <= 'F':             # <<<<<<<<<<<<<<
 *         return <int>(v) - 0x41 + 10  # ord('A') == 0x41
 *     elif 'a' <= v <= 'f':
 */
  }

  /* "yarl/_quoting.pyx":32
 *     elif 'A' <= v <= 'F':
 *         return <int>(v) - 0x41 + 10  # ord('A') == 0x41
 *     elif 'a' <= v <= 'f':             # <<<<<<<<<<<<<<
 *         return <int>(v) - 0x61 + 10  # ord('a') == 0x61
 *     else:
 */
  __pyx_t_1 = (97 <= __pyx_v_v);
  if (__pyx_t_1) {
    __pyx_t_1 = (__pyx_v_v <= 0x66);
  }
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "yarl/_quoting.pyx":33
 *         return <int>(v) - 0x41 + 10  # ord('A') == 0x41
 *     elif 'a' <= v <= 'f':
 *         return <int>(v) - 0x61 + 10  # ord('a') == 0x61             # <<<<<<<<<<<<<<
 *     else:
 *         return -1
 */
    __pyx_r = ((((int)__pyx_v_v) - 0x61) + 10);
    goto __pyx_L0;

    /* "yarl/_quoting.pyx":32
 *     elif 'A' <= v <= 'F':
 *         return <int>(v) - 0x41 + 10  # ord('A') == 0x41
 *     elif 'a' <= v <= 'f':             # <<<<<<<<<<<<<<
 *         return <int>(v) - 0x61 + 10  # ord('a') == 0x61
 *     else:
 */
  }

  /* "yarl/_quoting.pyx":35
 *         return <int>(v) - 0x61 + 10  # ord('a') == 0x61
 *     else:
 *         return -1             # <<<<<<<<<<<<<<
 * 
 * 
 */
  /*else*/ {
    __pyx_r = -1;
    goto __pyx_L0;
  }

  /* "yarl/_quoting.pyx":27
 * 
 * 
 * cdef inline int _from_hex(Py_UCS4 v):             # <<<<<<<<<<<<<<
 *     if '0' <= v <= '9':
 *         return <int>(v) - 0x30  # ord('0') == 0x30
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "yarl/_quoting.pyx":38
 * 
 * 
 * def _quote(val, *, str safe='', str protected='', bint qs=False, bint strict=True):             # <<<<<<<<<<<<<<
 *     if val is None:
 *         return None
 */

/* Python wrapper */
static PyObject *__pyx_pw_4yarl_8_quoting_1_quote(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_4yarl_8_quoting_1_quote = {"_quote", (PyCFunction)__pyx_pw_4yarl_8_quoting_1_quote, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_4yarl_8_quoting_1_quote(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_val = 0;
  PyObject *__pyx_v_safe = 0;
  PyObject *__pyx_v_protected = 0;
  int __pyx_v_qs;
  int __pyx_v_strict;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_quote (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_val,&__pyx_n_s_safe,&__pyx_n_s_protected,&__pyx_n_s_qs,&__pyx_n_s_strict,0};
    PyObject* values[5] = {0,0,0,0,0};
    values[1] = ((PyObject*)__pyx_kp_u_);
    values[2] = ((PyObject*)__pyx_kp_u_);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_val)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (kw_args > 0 && likely(kw_args <= 4)) {
        Py_ssize_t index;
        for (index = 1; index < 5 && kw_args > 0; index++) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, *__pyx_pyargnames[index]);
          if (value) { values[index] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_quote") < 0)) __PYX_ERR(0, 38, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_val = values[0];
    __pyx_v_safe = ((PyObject*)values[1]);
    __pyx_v_protected = ((PyObject*)values[2]);
    if (values[3]) {
      __pyx_v_qs = __Pyx_PyObject_IsTrue(values[3]); if (unlikely((__pyx_v_qs == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 38, __pyx_L3_error)
    } else {
      __pyx_v_qs = ((int)0);
    }
    if (values[4]) {
      __pyx_v_strict = __Pyx_PyObject_IsTrue(values[4]); if (unlikely((__pyx_v_strict == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 38, __pyx_L3_error)
    } else {
      __pyx_v_strict = ((int)1);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_quote", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 38, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("yarl._quoting._quote", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_safe), (&PyUnicode_Type), 1, "safe", 1))) __PYX_ERR(0, 38, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_protected), (&PyUnicode_Type), 1, "protected", 1))) __PYX_ERR(0, 38, __pyx_L1_error)
  __pyx_r = __pyx_pf_4yarl_8_quoting__quote(__pyx_self, __pyx_v_val, __pyx_v_safe, __pyx_v_protected, __pyx_v_qs, __pyx_v_strict);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4yarl_8_quoting__quote(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_val, PyObject *__pyx_v_safe, PyObject *__pyx_v_protected, int __pyx_v_qs, int __pyx_v_strict) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("_quote", 0);

  /* "yarl/_quoting.pyx":39
 * 
 * def _quote(val, *, str safe='', str protected='', bint qs=False, bint strict=True):
 *     if val is None:             # <<<<<<<<<<<<<<
 *         return None
 *     if not isinstance(val, str):
 */
  __pyx_t_1 = (__pyx_v_val == Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "yarl/_quoting.pyx":40
 * def _quote(val, *, str safe='', str protected='', bint qs=False, bint strict=True):
 *     if val is None:
 *         return None             # <<<<<<<<<<<<<<
 *     if not isinstance(val, str):
 *         raise TypeError("Argument should be str")
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_None);
    __pyx_r = Py_None;
    goto __pyx_L0;

    /* "yarl/_quoting.pyx":39
 * 
 * def _quote(val, *, str safe='', str protected='', bint qs=False, bint strict=True):
 *     if val is None:             # <<<<<<<<<<<<<<
 *         return None
 *     if not isinstance(val, str):
 */
  }

  /* "yarl/_quoting.pyx":41
 *     if val is None:
 *         return None
 *     if not isinstance(val, str):             # <<<<<<<<<<<<<<
 *         raise TypeError("Argument should be str")
 *     if not val:
 */
  __pyx_t_2 = PyUnicode_Check(__pyx_v_val); 
  __pyx_t_1 = ((!(__pyx_t_2 != 0)) != 0);
  if (__pyx_t_1) {

    /* "yarl/_quoting.pyx":42
 *         return None
 *     if not isinstance(val, str):
 *         raise TypeError("Argument should be str")             # <<<<<<<<<<<<<<
 *     if not val:
 *         return ''
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 42, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 42, __pyx_L1_error)

    /* "yarl/_quoting.pyx":41
 *     if val is None:
 *         return None
 *     if not isinstance(val, str):             # <<<<<<<<<<<<<<
 *         raise TypeError("Argument should be str")
 *     if not val:
 */
  }

  /* "yarl/_quoting.pyx":43
 *     if not isinstance(val, str):
 *         raise TypeError("Argument should be str")
 *     if not val:             # <<<<<<<<<<<<<<
 *         return ''
 *     return _do_quote(<str>val, safe, protected, qs, strict)
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_val); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 43, __pyx_L1_error)
  __pyx_t_2 = ((!__pyx_t_1) != 0);
  if (__pyx_t_2) {

    /* "yarl/_quoting.pyx":44
 *         raise TypeError("Argument should be str")
 *     if not val:
 *         return ''             # <<<<<<<<<<<<<<
 *     return _do_quote(<str>val, safe, protected, qs, strict)
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_);
    __pyx_r = __pyx_kp_u_;
    goto __pyx_L0;

    /* "yarl/_quoting.pyx":43
 *     if not isinstance(val, str):
 *         raise TypeError("Argument should be str")
 *     if not val:             # <<<<<<<<<<<<<<
 *         return ''
 *     return _do_quote(<str>val, safe, protected, qs, strict)
 */
  }

  /* "yarl/_quoting.pyx":45
 *     if not val:
 *         return ''
 *     return _do_quote(<str>val, safe, protected, qs, strict)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = __pyx_f_4yarl_8_quoting__do_quote(((PyObject*)__pyx_v_val), __pyx_v_safe, __pyx_v_protected, __pyx_v_qs, __pyx_v_strict); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 45, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "yarl/_quoting.pyx":38
 * 
 * 
 * def _quote(val, *, str safe='', str protected='', bint qs=False, bint strict=True):             # <<<<<<<<<<<<<<
 *     if val is None:
 *         return None
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("yarl._quoting._quote", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "yarl/_quoting.pyx":48
 * 
 * 
 * cdef str _do_quote(str val, str safe, str protected, bint qs, bint strict):             # <<<<<<<<<<<<<<
 *     cdef uint8_t b
 *     cdef Py_UCS4 ch, unquoted
 */

static PyObject *__pyx_f_4yarl_8_quoting__do_quote(PyObject *__pyx_v_val, PyObject *__pyx_v_safe, PyObject *__pyx_v_protected, int __pyx_v_qs, int __pyx_v_strict) {
  uint8_t __pyx_v_b;
  Py_UCS4 __pyx_v_ch;
  Py_ssize_t __pyx_v_ret_size;
  PyObject *__pyx_v_ret = 0;
  Py_ssize_t __pyx_v_ret_idx;
  int __pyx_v_has_pct;
  Py_UCS4 __pyx_v_pct[2];
  int __pyx_v_digit1;
  int __pyx_v_digit2;
  int __pyx_v_idx;
  PyObject *__pyx_v_ch_bytes = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  Py_ssize_t __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  Py_UCS4 __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *(*__pyx_t_12)(PyObject *);
  uint8_t __pyx_t_13;
  __Pyx_RefNannySetupContext("_do_quote", 0);
  __Pyx_INCREF(__pyx_v_safe);

  /* "yarl/_quoting.pyx":55
 *     # UTF8 may take up to 5 bytes per symbol
 *     # every byte is encoded as %XX -- 3 bytes
 *     cdef Py_ssize_t ret_size = len(val)*3*5 + 1             # <<<<<<<<<<<<<<
 *     cdef object ret = PyUnicode_New(ret_size, 1114111)
 *     cdef Py_ssize_t ret_idx = 0
 */
  if (unlikely(__pyx_v_val == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 55, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyUnicode_GET_LENGTH(__pyx_v_val); if (unlikely(__pyx_t_1 == -1)) __PYX_ERR(0, 55, __pyx_L1_error)
  __pyx_v_ret_size = (((__pyx_t_1 * 3) * 5) + 1);

  /* "yarl/_quoting.pyx":56
 *     # every byte is encoded as %XX -- 3 bytes
 *     cdef Py_ssize_t ret_size = len(val)*3*5 + 1
 *     cdef object ret = PyUnicode_New(ret_size, 1114111)             # <<<<<<<<<<<<<<
 *     cdef Py_ssize_t ret_idx = 0
 *     cdef int has_pct = 0
 */
  __pyx_t_2 = PyUnicode_New(__pyx_v_ret_size, 0x10FFFF); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 56, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_ret = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "yarl/_quoting.pyx":57
 *     cdef Py_ssize_t ret_size = len(val)*3*5 + 1
 *     cdef object ret = PyUnicode_New(ret_size, 1114111)
 *     cdef Py_ssize_t ret_idx = 0             # <<<<<<<<<<<<<<
 *     cdef int has_pct = 0
 *     cdef Py_UCS4 pct[2]
 */
  __pyx_v_ret_idx = 0;

  /* "yarl/_quoting.pyx":58
 *     cdef object ret = PyUnicode_New(ret_size, 1114111)
 *     cdef Py_ssize_t ret_idx = 0
 *     cdef int has_pct = 0             # <<<<<<<<<<<<<<
 *     cdef Py_UCS4 pct[2]
 *     cdef int digit1, digit2
 */
  __pyx_v_has_pct = 0;

  /* "yarl/_quoting.pyx":61
 *     cdef Py_UCS4 pct[2]
 *     cdef int digit1, digit2
 *     safe += ALLOWED             # <<<<<<<<<<<<<<
 *     if not qs:
 *         safe += '+&=;'
 */
  __pyx_t_2 = __Pyx_PyUnicode_ConcatSafe(__pyx_v_safe, __pyx_v_4yarl_8_quoting_ALLOWED); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 61, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF_SET(__pyx_v_safe, ((PyObject*)__pyx_t_2));
  __pyx_t_2 = 0;

  /* "yarl/_quoting.pyx":62
 *     cdef int digit1, digit2
 *     safe += ALLOWED
 *     if not qs:             # <<<<<<<<<<<<<<
 *         safe += '+&=;'
 *     safe += protected
 */
  __pyx_t_3 = ((!(__pyx_v_qs != 0)) != 0);
  if (__pyx_t_3) {

    /* "yarl/_quoting.pyx":63
 *     safe += ALLOWED
 *     if not qs:
 *         safe += '+&=;'             # <<<<<<<<<<<<<<
 *     safe += protected
 *     cdef int idx = 0
 */
    __pyx_t_2 = __Pyx_PyUnicode_Concat(__pyx_v_safe, __pyx_kp_u__3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 63, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF_SET(__pyx_v_safe, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "yarl/_quoting.pyx":62
 *     cdef int digit1, digit2
 *     safe += ALLOWED
 *     if not qs:             # <<<<<<<<<<<<<<
 *         safe += '+&=;'
 *     safe += protected
 */
  }

  /* "yarl/_quoting.pyx":64
 *     if not qs:
 *         safe += '+&=;'
 *     safe += protected             # <<<<<<<<<<<<<<
 *     cdef int idx = 0
 *     while idx < len(val):
 */
  __pyx_t_2 = __Pyx_PyUnicode_ConcatSafe(__pyx_v_safe, __pyx_v_protected); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 64, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF_SET(__pyx_v_safe, ((PyObject*)__pyx_t_2));
  __pyx_t_2 = 0;

  /* "yarl/_quoting.pyx":65
 *         safe += '+&=;'
 *     safe += protected
 *     cdef int idx = 0             # <<<<<<<<<<<<<<
 *     while idx < len(val):
 *         ch = val[idx]
 */
  __pyx_v_idx = 0;

  /* "yarl/_quoting.pyx":66
 *     safe += protected
 *     cdef int idx = 0
 *     while idx < len(val):             # <<<<<<<<<<<<<<
 *         ch = val[idx]
 *         idx += 1
 */
  while (1) {
    if (unlikely(__pyx_v_val == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 66, __pyx_L1_error)
    }
    __pyx_t_1 = __Pyx_PyUnicode_GET_LENGTH(__pyx_v_val); if (unlikely(__pyx_t_1 == -1)) __PYX_ERR(0, 66, __pyx_L1_error)
    __pyx_t_3 = ((__pyx_v_idx < __pyx_t_1) != 0);
    if (!__pyx_t_3) break;

    /* "yarl/_quoting.pyx":67
 *     cdef int idx = 0
 *     while idx < len(val):
 *         ch = val[idx]             # <<<<<<<<<<<<<<
 *         idx += 1
 * 
 */
    __pyx_t_4 = __Pyx_GetItemInt_Unicode(__pyx_v_val, __pyx_v_idx, int, 1, __Pyx_PyInt_From_int, 0, 1, 1); if (unlikely(__pyx_t_4 == (Py_UCS4)-1)) __PYX_ERR(0, 67, __pyx_L1_error)
    __pyx_v_ch = __pyx_t_4;

    /* "yarl/_quoting.pyx":68
 *     while idx < len(val):
 *         ch = val[idx]
 *         idx += 1             # <<<<<<<<<<<<<<
 * 
 *         if has_pct:
 */
    __pyx_v_idx = (__pyx_v_idx + 1);

    /* "yarl/_quoting.pyx":70
 *         idx += 1
 * 
 *         if has_pct:             # <<<<<<<<<<<<<<
 *             pct[has_pct-1] = ch
 *             has_pct += 1
 */
    __pyx_t_3 = (__pyx_v_has_pct != 0);
    if (__pyx_t_3) {

      /* "yarl/_quoting.pyx":71
 * 
 *         if has_pct:
 *             pct[has_pct-1] = ch             # <<<<<<<<<<<<<<
 *             has_pct += 1
 *             if has_pct == 3:
 */
      (__pyx_v_pct[(__pyx_v_has_pct - 1)]) = __pyx_v_ch;

      /* "yarl/_quoting.pyx":72
 *         if has_pct:
 *             pct[has_pct-1] = ch
 *             has_pct += 1             # <<<<<<<<<<<<<<
 *             if has_pct == 3:
 *                 digit1 = _from_hex(pct[0])
 */
      __pyx_v_has_pct = (__pyx_v_has_pct + 1);

      /* "yarl/_quoting.pyx":73
 *             pct[has_pct-1] = ch
 *             has_pct += 1
 *             if has_pct == 3:             # <<<<<<<<<<<<<<
 *                 digit1 = _from_hex(pct[0])
 *                 digit2 = _from_hex(pct[1])
 */
      __pyx_t_3 = ((__pyx_v_has_pct == 3) != 0);
      if (__pyx_t_3) {

        /* "yarl/_quoting.pyx":74
 *             has_pct += 1
 *             if has_pct == 3:
 *                 digit1 = _from_hex(pct[0])             # <<<<<<<<<<<<<<
 *                 digit2 = _from_hex(pct[1])
 *                 if digit1 == -1 or digit2 == -1:
 */
        __pyx_v_digit1 = __pyx_f_4yarl_8_quoting__from_hex((__pyx_v_pct[0]));

        /* "yarl/_quoting.pyx":75
 *             if has_pct == 3:
 *                 digit1 = _from_hex(pct[0])
 *                 digit2 = _from_hex(pct[1])             # <<<<<<<<<<<<<<
 *                 if digit1 == -1 or digit2 == -1:
 *                     if strict:
 */
        __pyx_v_digit2 = __pyx_f_4yarl_8_quoting__from_hex((__pyx_v_pct[1]));

        /* "yarl/_quoting.pyx":76
 *                 digit1 = _from_hex(pct[0])
 *                 digit2 = _from_hex(pct[1])
 *                 if digit1 == -1 or digit2 == -1:             # <<<<<<<<<<<<<<
 *                     if strict:
 *                         raise ValueError("Unallowed PCT %{}{}".format(pct[0],
 */
        __pyx_t_5 = ((__pyx_v_digit1 == -1L) != 0);
        if (!__pyx_t_5) {
        } else {
          __pyx_t_3 = __pyx_t_5;
          goto __pyx_L9_bool_binop_done;
        }
        __pyx_t_5 = ((__pyx_v_digit2 == -1L) != 0);
        __pyx_t_3 = __pyx_t_5;
        __pyx_L9_bool_binop_done:;
        if (__pyx_t_3) {

          /* "yarl/_quoting.pyx":77
 *                 digit2 = _from_hex(pct[1])
 *                 if digit1 == -1 or digit2 == -1:
 *                     if strict:             # <<<<<<<<<<<<<<
 *                         raise ValueError("Unallowed PCT %{}{}".format(pct[0],
 *                                                                       pct[1]))
 */
          __pyx_t_3 = (__pyx_v_strict != 0);
          if (__pyx_t_3) {

            /* "yarl/_quoting.pyx":78
 *                 if digit1 == -1 or digit2 == -1:
 *                     if strict:
 *                         raise ValueError("Unallowed PCT %{}{}".format(pct[0],             # <<<<<<<<<<<<<<
 *                                                                       pct[1]))
 *                     else:
 */
            __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_Unallowed_PCT, __pyx_n_s_format); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 78, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_6);
            __pyx_t_7 = PyUnicode_FromOrdinal((__pyx_v_pct[0])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 78, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_7);

            /* "yarl/_quoting.pyx":79
 *                     if strict:
 *                         raise ValueError("Unallowed PCT %{}{}".format(pct[0],
 *                                                                       pct[1]))             # <<<<<<<<<<<<<<
 *                     else:
 *                         PyUnicode_WriteChar(ret, ret_idx, '%')
 */
            __pyx_t_8 = PyUnicode_FromOrdinal((__pyx_v_pct[1])); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 79, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_8);
            __pyx_t_9 = NULL;
            __pyx_t_10 = 0;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
              __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_6);
              if (likely(__pyx_t_9)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
                __Pyx_INCREF(__pyx_t_9);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_6, function);
                __pyx_t_10 = 1;
              }
            }
            #if CYTHON_FAST_PYCALL
            if (PyFunction_Check(__pyx_t_6)) {
              PyObject *__pyx_temp[3] = {__pyx_t_9, __pyx_t_7, __pyx_t_8};
              __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 78, __pyx_L1_error)
              __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
              __Pyx_GOTREF(__pyx_t_2);
              __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
              __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
            } else
            #endif
            #if CYTHON_FAST_PYCCALL
            if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
              PyObject *__pyx_temp[3] = {__pyx_t_9, __pyx_t_7, __pyx_t_8};
              __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 78, __pyx_L1_error)
              __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
              __Pyx_GOTREF(__pyx_t_2);
              __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
              __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
            } else
            #endif
            {
              __pyx_t_11 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 78, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_11);
              if (__pyx_t_9) {
                __Pyx_GIVEREF(__pyx_t_9); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_9); __pyx_t_9 = NULL;
              }
              __Pyx_GIVEREF(__pyx_t_7);
              PyTuple_SET_ITEM(__pyx_t_11, 0+__pyx_t_10, __pyx_t_7);
              __Pyx_GIVEREF(__pyx_t_8);
              PyTuple_SET_ITEM(__pyx_t_11, 1+__pyx_t_10, __pyx_t_8);
              __pyx_t_7 = 0;
              __pyx_t_8 = 0;
              __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_11, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 78, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_2);
              __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
            }
            __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

            /* "yarl/_quoting.pyx":78
 *                 if digit1 == -1 or digit2 == -1:
 *                     if strict:
 *                         raise ValueError("Unallowed PCT %{}{}".format(pct[0],             # <<<<<<<<<<<<<<
 *                                                                       pct[1]))
 *                     else:
 */
            __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 78, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_6);
            __Pyx_GIVEREF(__pyx_t_2);
            PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_2);
            __pyx_t_2 = 0;
            __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 78, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_2);
            __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
            __Pyx_Raise(__pyx_t_2, 0, 0, 0);
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            __PYX_ERR(0, 78, __pyx_L1_error)

            /* "yarl/_quoting.pyx":77
 *                 digit2 = _from_hex(pct[1])
 *                 if digit1 == -1 or digit2 == -1:
 *                     if strict:             # <<<<<<<<<<<<<<
 *                         raise ValueError("Unallowed PCT %{}{}".format(pct[0],
 *                                                                       pct[1]))
 */
          }

          /* "yarl/_quoting.pyx":81
 *                                                                       pct[1]))
 *                     else:
 *                         PyUnicode_WriteChar(ret, ret_idx, '%')             # <<<<<<<<<<<<<<
 *                         ret_idx += 1
 *                         PyUnicode_WriteChar(ret, ret_idx, '2')
 */
          /*else*/ {
            PyUnicode_WriteChar(__pyx_v_ret, __pyx_v_ret_idx, 37);

            /* "yarl/_quoting.pyx":82
 *                     else:
 *                         PyUnicode_WriteChar(ret, ret_idx, '%')
 *                         ret_idx += 1             # <<<<<<<<<<<<<<
 *                         PyUnicode_WriteChar(ret, ret_idx, '2')
 *                         ret_idx += 1
 */
            __pyx_v_ret_idx = (__pyx_v_ret_idx + 1);

            /* "yarl/_quoting.pyx":83
 *                         PyUnicode_WriteChar(ret, ret_idx, '%')
 *                         ret_idx += 1
 *                         PyUnicode_WriteChar(ret, ret_idx, '2')             # <<<<<<<<<<<<<<
 *                         ret_idx += 1
 *                         PyUnicode_WriteChar(ret, ret_idx, '5')
 */
            PyUnicode_WriteChar(__pyx_v_ret, __pyx_v_ret_idx, 50);

            /* "yarl/_quoting.pyx":84
 *                         ret_idx += 1
 *                         PyUnicode_WriteChar(ret, ret_idx, '2')
 *                         ret_idx += 1             # <<<<<<<<<<<<<<
 *                         PyUnicode_WriteChar(ret, ret_idx, '5')
 *                         ret_idx += 1
 */
            __pyx_v_ret_idx = (__pyx_v_ret_idx + 1);

            /* "yarl/_quoting.pyx":85
 *                         PyUnicode_WriteChar(ret, ret_idx, '2')
 *                         ret_idx += 1
 *                         PyUnicode_WriteChar(ret, ret_idx, '5')             # <<<<<<<<<<<<<<
 *                         ret_idx += 1
 *                         idx -= 2
 */
            PyUnicode_WriteChar(__pyx_v_ret, __pyx_v_ret_idx, 53);

            /* "yarl/_quoting.pyx":86
 *                         ret_idx += 1
 *                         PyUnicode_WriteChar(ret, ret_idx, '5')
 *                         ret_idx += 1             # <<<<<<<<<<<<<<
 *                         idx -= 2
 *                         has_pct = 0
 */
            __pyx_v_ret_idx = (__pyx_v_ret_idx + 1);

            /* "yarl/_quoting.pyx":87
 *                         PyUnicode_WriteChar(ret, ret_idx, '5')
 *                         ret_idx += 1
 *                         idx -= 2             # <<<<<<<<<<<<<<
 *                         has_pct = 0
 *                         continue
 */
            __pyx_v_idx = (__pyx_v_idx - 2);

            /* "yarl/_quoting.pyx":88
 *                         ret_idx += 1
 *                         idx -= 2
 *                         has_pct = 0             # <<<<<<<<<<<<<<
 *                         continue
 * 
 */
            __pyx_v_has_pct = 0;

            /* "yarl/_quoting.pyx":89
 *                         idx -= 2
 *                         has_pct = 0
 *                         continue             # <<<<<<<<<<<<<<
 * 
 *                 ch = <Py_UCS4>(digit1 << 4 | digit2)
 */
            goto __pyx_L4_continue;
          }

          /* "yarl/_quoting.pyx":76
 *                 digit1 = _from_hex(pct[0])
 *                 digit2 = _from_hex(pct[1])
 *                 if digit1 == -1 or digit2 == -1:             # <<<<<<<<<<<<<<
 *                     if strict:
 *                         raise ValueError("Unallowed PCT %{}{}".format(pct[0],
 */
        }

        /* "yarl/_quoting.pyx":91
 *                         continue
 * 
 *                 ch = <Py_UCS4>(digit1 << 4 | digit2)             # <<<<<<<<<<<<<<
 *                 has_pct = 0
 * 
 */
        __pyx_v_ch = ((Py_UCS4)((__pyx_v_digit1 << 4) | __pyx_v_digit2));

        /* "yarl/_quoting.pyx":92
 * 
 *                 ch = <Py_UCS4>(digit1 << 4 | digit2)
 *                 has_pct = 0             # <<<<<<<<<<<<<<
 * 
 *                 if ch in protected:
 */
        __pyx_v_has_pct = 0;

        /* "yarl/_quoting.pyx":94
 *                 has_pct = 0
 * 
 *                 if ch in protected:             # <<<<<<<<<<<<<<
 *                     PyUnicode_WriteChar(ret, ret_idx, '%')
 *                     ret_idx += 1
 */
        if (unlikely(__pyx_v_protected == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "argument of type 'NoneType' is not iterable");
          __PYX_ERR(0, 94, __pyx_L1_error)
        }
        __pyx_t_3 = ((__Pyx_UnicodeContainsUCS4(__pyx_v_protected, __pyx_v_ch)) != 0);
        if (__pyx_t_3) {

          /* "yarl/_quoting.pyx":95
 * 
 *                 if ch in protected:
 *                     PyUnicode_WriteChar(ret, ret_idx, '%')             # <<<<<<<<<<<<<<
 *                     ret_idx += 1
 *                     PyUnicode_WriteChar(ret, ret_idx, _hex(<uint8_t>ch >> 4))
 */
          PyUnicode_WriteChar(__pyx_v_ret, __pyx_v_ret_idx, 37);

          /* "yarl/_quoting.pyx":96
 *                 if ch in protected:
 *                     PyUnicode_WriteChar(ret, ret_idx, '%')
 *                     ret_idx += 1             # <<<<<<<<<<<<<<
 *                     PyUnicode_WriteChar(ret, ret_idx, _hex(<uint8_t>ch >> 4))
 *                     ret_idx += 1
 */
          __pyx_v_ret_idx = (__pyx_v_ret_idx + 1);

          /* "yarl/_quoting.pyx":97
 *                     PyUnicode_WriteChar(ret, ret_idx, '%')
 *                     ret_idx += 1
 *                     PyUnicode_WriteChar(ret, ret_idx, _hex(<uint8_t>ch >> 4))             # <<<<<<<<<<<<<<
 *                     ret_idx += 1
 *                     PyUnicode_WriteChar(ret, ret_idx, _hex(<uint8_t>ch & 0x0f))
 */
          PyUnicode_WriteChar(__pyx_v_ret, __pyx_v_ret_idx, __pyx_f_4yarl_8_quoting__hex((((uint8_t)__pyx_v_ch) >> 4)));

          /* "yarl/_quoting.pyx":98
 *                     ret_idx += 1
 *                     PyUnicode_WriteChar(ret, ret_idx, _hex(<uint8_t>ch >> 4))
 *                     ret_idx += 1             # <<<<<<<<<<<<<<
 *                     PyUnicode_WriteChar(ret, ret_idx, _hex(<uint8_t>ch & 0x0f))
 *                     ret_idx += 1
 */
          __pyx_v_ret_idx = (__pyx_v_ret_idx + 1);

          /* "yarl/_quoting.pyx":99
 *                     PyUnicode_WriteChar(ret, ret_idx, _hex(<uint8_t>ch >> 4))
 *                     ret_idx += 1
 *                     PyUnicode_WriteChar(ret, ret_idx, _hex(<uint8_t>ch & 0x0f))             # <<<<<<<<<<<<<<
 *                     ret_idx += 1
 *                 elif ch in safe:
 */
          PyUnicode_WriteChar(__pyx_v_ret, __pyx_v_ret_idx, __pyx_f_4yarl_8_quoting__hex((((uint8_t)__pyx_v_ch) & 0x0f)));

          /* "yarl/_quoting.pyx":100
 *                     ret_idx += 1
 *                     PyUnicode_WriteChar(ret, ret_idx, _hex(<uint8_t>ch & 0x0f))
 *                     ret_idx += 1             # <<<<<<<<<<<<<<
 *                 elif ch in safe:
 *                     PyUnicode_WriteChar(ret, ret_idx, ch)
 */
          __pyx_v_ret_idx = (__pyx_v_ret_idx + 1);

          /* "yarl/_quoting.pyx":94
 *                 has_pct = 0
 * 
 *                 if ch in protected:             # <<<<<<<<<<<<<<
 *                     PyUnicode_WriteChar(ret, ret_idx, '%')
 *                     ret_idx += 1
 */
          goto __pyx_L12;
        }

        /* "yarl/_quoting.pyx":101
 *                     PyUnicode_WriteChar(ret, ret_idx, _hex(<uint8_t>ch & 0x0f))
 *                     ret_idx += 1
 *                 elif ch in safe:             # <<<<<<<<<<<<<<
 *                     PyUnicode_WriteChar(ret, ret_idx, ch)
 *                     ret_idx += 1
 */
        __pyx_t_3 = ((__Pyx_UnicodeContainsUCS4(__pyx_v_safe, __pyx_v_ch)) != 0);
        if (__pyx_t_3) {

          /* "yarl/_quoting.pyx":102
 *                     ret_idx += 1
 *                 elif ch in safe:
 *                     PyUnicode_WriteChar(ret, ret_idx, ch)             # <<<<<<<<<<<<<<
 *                     ret_idx += 1
 *                 else:
 */
          PyUnicode_WriteChar(__pyx_v_ret, __pyx_v_ret_idx, __pyx_v_ch);

          /* "yarl/_quoting.pyx":103
 *                 elif ch in safe:
 *                     PyUnicode_WriteChar(ret, ret_idx, ch)
 *                     ret_idx += 1             # <<<<<<<<<<<<<<
 *                 else:
 *                     PyUnicode_WriteChar(ret, ret_idx, '%')
 */
          __pyx_v_ret_idx = (__pyx_v_ret_idx + 1);

          /* "yarl/_quoting.pyx":101
 *                     PyUnicode_WriteChar(ret, ret_idx, _hex(<uint8_t>ch & 0x0f))
 *                     ret_idx += 1
 *                 elif ch in safe:             # <<<<<<<<<<<<<<
 *                     PyUnicode_WriteChar(ret, ret_idx, ch)
 *                     ret_idx += 1
 */
          goto __pyx_L12;
        }

        /* "yarl/_quoting.pyx":105
 *                     ret_idx += 1
 *                 else:
 *                     PyUnicode_WriteChar(ret, ret_idx, '%')             # <<<<<<<<<<<<<<
 *                     ret_idx += 1
 *                     PyUnicode_WriteChar(ret, ret_idx, _hex(<uint8_t>ch >> 4))
 */
        /*else*/ {
          PyUnicode_WriteChar(__pyx_v_ret, __pyx_v_ret_idx, 37);

          /* "yarl/_quoting.pyx":106
 *                 else:
 *                     PyUnicode_WriteChar(ret, ret_idx, '%')
 *                     ret_idx += 1             # <<<<<<<<<<<<<<
 *                     PyUnicode_WriteChar(ret, ret_idx, _hex(<uint8_t>ch >> 4))
 *                     ret_idx += 1
 */
          __pyx_v_ret_idx = (__pyx_v_ret_idx + 1);

          /* "yarl/_quoting.pyx":107
 *                     PyUnicode_WriteChar(ret, ret_idx, '%')
 *                     ret_idx += 1
 *                     PyUnicode_WriteChar(ret, ret_idx, _hex(<uint8_t>ch >> 4))             # <<<<<<<<<<<<<<
 *                     ret_idx += 1
 *                     PyUnicode_WriteChar(ret, ret_idx, _hex(<uint8_t>ch & 0x0f))
 */
          PyUnicode_WriteChar(__pyx_v_ret, __pyx_v_ret_idx, __pyx_f_4yarl_8_quoting__hex((((uint8_t)__pyx_v_ch) >> 4)));

          /* "yarl/_quoting.pyx":108
 *                     ret_idx += 1
 *                     PyUnicode_WriteChar(ret, ret_idx, _hex(<uint8_t>ch >> 4))
 *                     ret_idx += 1             # <<<<<<<<<<<<<<
 *                     PyUnicode_WriteChar(ret, ret_idx, _hex(<uint8_t>ch & 0x0f))
 *                     ret_idx += 1
 */
          __pyx_v_ret_idx = (__pyx_v_ret_idx + 1);

          /* "yarl/_quoting.pyx":109
 *                     PyUnicode_WriteChar(ret, ret_idx, _hex(<uint8_t>ch >> 4))
 *                     ret_idx += 1
 *                     PyUnicode_WriteChar(ret, ret_idx, _hex(<uint8_t>ch & 0x0f))             # <<<<<<<<<<<<<<
 *                     ret_idx += 1
 * 
 */
          PyUnicode_WriteChar(__pyx_v_ret, __pyx_v_ret_idx, __pyx_f_4yarl_8_quoting__hex((((uint8_t)__pyx_v_ch) & 0x0f)));

          /* "yarl/_quoting.pyx":110
 *                     ret_idx += 1
 *                     PyUnicode_WriteChar(ret, ret_idx, _hex(<uint8_t>ch & 0x0f))
 *                     ret_idx += 1             # <<<<<<<<<<<<<<
 * 
 *             # special case, if we have only one char after "%"
 */
          __pyx_v_ret_idx = (__pyx_v_ret_idx + 1);
        }
        __pyx_L12:;

        /* "yarl/_quoting.pyx":73
 *             pct[has_pct-1] = ch
 *             has_pct += 1
 *             if has_pct == 3:             # <<<<<<<<<<<<<<
 *                 digit1 = _from_hex(pct[0])
 *                 digit2 = _from_hex(pct[1])
 */
        goto __pyx_L7;
      }

      /* "yarl/_quoting.pyx":113
 * 
 *             # special case, if we have only one char after "%"
 *             elif has_pct == 2 and idx == len(val):             # <<<<<<<<<<<<<<
 *                 if strict:
 *                     raise ValueError("Unallowed PCT %{}".format(pct[0]))
 */
      __pyx_t_5 = ((__pyx_v_has_pct == 2) != 0);
      if (__pyx_t_5) {
      } else {
        __pyx_t_3 = __pyx_t_5;
        goto __pyx_L13_bool_binop_done;
      }
      if (unlikely(__pyx_v_val == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
        __PYX_ERR(0, 113, __pyx_L1_error)
      }
      __pyx_t_1 = __Pyx_PyUnicode_GET_LENGTH(__pyx_v_val); if (unlikely(__pyx_t_1 == -1)) __PYX_ERR(0, 113, __pyx_L1_error)
      __pyx_t_5 = ((__pyx_v_idx == __pyx_t_1) != 0);
      __pyx_t_3 = __pyx_t_5;
      __pyx_L13_bool_binop_done:;
      if (__pyx_t_3) {

        /* "yarl/_quoting.pyx":114
 *             # special case, if we have only one char after "%"
 *             elif has_pct == 2 and idx == len(val):
 *                 if strict:             # <<<<<<<<<<<<<<
 *                     raise ValueError("Unallowed PCT %{}".format(pct[0]))
 *                 else:
 */
        __pyx_t_3 = (__pyx_v_strict != 0);
        if (__pyx_t_3) {

          /* "yarl/_quoting.pyx":115
 *             elif has_pct == 2 and idx == len(val):
 *                 if strict:
 *                     raise ValueError("Unallowed PCT %{}".format(pct[0]))             # <<<<<<<<<<<<<<
 *                 else:
 *                     PyUnicode_WriteChar(ret, ret_idx, '%')
 */
          __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_kp_u_Unallowed_PCT_2, __pyx_n_s_format); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 115, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_t_11 = PyUnicode_FromOrdinal((__pyx_v_pct[0])); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 115, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_11);
          __pyx_t_8 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
            __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_6);
            if (likely(__pyx_t_8)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
              __Pyx_INCREF(__pyx_t_8);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_6, function);
            }
          }
          if (!__pyx_t_8) {
            __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_11); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 115, __pyx_L1_error)
            __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
            __Pyx_GOTREF(__pyx_t_2);
          } else {
            #if CYTHON_FAST_PYCALL
            if (PyFunction_Check(__pyx_t_6)) {
              PyObject *__pyx_temp[2] = {__pyx_t_8, __pyx_t_11};
              __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_6, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 115, __pyx_L1_error)
              __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
              __Pyx_GOTREF(__pyx_t_2);
              __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
            } else
            #endif
            #if CYTHON_FAST_PYCCALL
            if (__Pyx_PyFastCFunction_Check(__pyx_t_6)) {
              PyObject *__pyx_temp[2] = {__pyx_t_8, __pyx_t_11};
              __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_6, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 115, __pyx_L1_error)
              __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
              __Pyx_GOTREF(__pyx_t_2);
              __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
            } else
            #endif
            {
              __pyx_t_7 = PyTuple_New(1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 115, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_7);
              __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_8); __pyx_t_8 = NULL;
              __Pyx_GIVEREF(__pyx_t_11);
              PyTuple_SET_ITEM(__pyx_t_7, 0+1, __pyx_t_11);
              __pyx_t_11 = 0;
              __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_t_7, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 115, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_2);
              __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
            }
          }
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 115, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          __Pyx_GIVEREF(__pyx_t_2);
          PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_2);
          __pyx_t_2 = 0;
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 115, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_Raise(__pyx_t_2, 0, 0, 0);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __PYX_ERR(0, 115, __pyx_L1_error)

          /* "yarl/_quoting.pyx":114
 *             # special case, if we have only one char after "%"
 *             elif has_pct == 2 and idx == len(val):
 *                 if strict:             # <<<<<<<<<<<<<<
 *                     raise ValueError("Unallowed PCT %{}".format(pct[0]))
 *                 else:
 */
        }

        /* "yarl/_quoting.pyx":117
 *                     raise ValueError("Unallowed PCT %{}".format(pct[0]))
 *                 else:
 *                     PyUnicode_WriteChar(ret, ret_idx, '%')             # <<<<<<<<<<<<<<
 *                     ret_idx += 1
 *                     PyUnicode_WriteChar(ret, ret_idx, '2')
 */
        /*else*/ {
          PyUnicode_WriteChar(__pyx_v_ret, __pyx_v_ret_idx, 37);

          /* "yarl/_quoting.pyx":118
 *                 else:
 *                     PyUnicode_WriteChar(ret, ret_idx, '%')
 *                     ret_idx += 1             # <<<<<<<<<<<<<<
 *                     PyUnicode_WriteChar(ret, ret_idx, '2')
 *                     ret_idx += 1
 */
          __pyx_v_ret_idx = (__pyx_v_ret_idx + 1);

          /* "yarl/_quoting.pyx":119
 *                     PyUnicode_WriteChar(ret, ret_idx, '%')
 *                     ret_idx += 1
 *                     PyUnicode_WriteChar(ret, ret_idx, '2')             # <<<<<<<<<<<<<<
 *                     ret_idx += 1
 *                     PyUnicode_WriteChar(ret, ret_idx, '5')
 */
          PyUnicode_WriteChar(__pyx_v_ret, __pyx_v_ret_idx, 50);

          /* "yarl/_quoting.pyx":120
 *                     ret_idx += 1
 *                     PyUnicode_WriteChar(ret, ret_idx, '2')
 *                     ret_idx += 1             # <<<<<<<<<<<<<<
 *                     PyUnicode_WriteChar(ret, ret_idx, '5')
 *                     ret_idx += 1
 */
          __pyx_v_ret_idx = (__pyx_v_ret_idx + 1);

          /* "yarl/_quoting.pyx":121
 *                     PyUnicode_WriteChar(ret, ret_idx, '2')
 *                     ret_idx += 1
 *                     PyUnicode_WriteChar(ret, ret_idx, '5')             # <<<<<<<<<<<<<<
 *                     ret_idx += 1
 * 
 */
          PyUnicode_WriteChar(__pyx_v_ret, __pyx_v_ret_idx, 53);

          /* "yarl/_quoting.pyx":122
 *                     ret_idx += 1
 *                     PyUnicode_WriteChar(ret, ret_idx, '5')
 *                     ret_idx += 1             # <<<<<<<<<<<<<<
 * 
 *                     idx -= 1
 */
          __pyx_v_ret_idx = (__pyx_v_ret_idx + 1);

          /* "yarl/_quoting.pyx":124
 *                     ret_idx += 1
 * 
 *                     idx -= 1             # <<<<<<<<<<<<<<
 *                     has_pct = 0
 * 
 */
          __pyx_v_idx = (__pyx_v_idx - 1);

          /* "yarl/_quoting.pyx":125
 * 
 *                     idx -= 1
 *                     has_pct = 0             # <<<<<<<<<<<<<<
 * 
 *             continue
 */
          __pyx_v_has_pct = 0;
        }

        /* "yarl/_quoting.pyx":113
 * 
 *             # special case, if we have only one char after "%"
 *             elif has_pct == 2 and idx == len(val):             # <<<<<<<<<<<<<<
 *                 if strict:
 *                     raise ValueError("Unallowed PCT %{}".format(pct[0]))
 */
      }
      __pyx_L7:;

      /* "yarl/_quoting.pyx":127
 *                     has_pct = 0
 * 
 *             continue             # <<<<<<<<<<<<<<
 * 
 *         elif ch == '%':
 */
      goto __pyx_L4_continue;

      /* "yarl/_quoting.pyx":70
 *         idx += 1
 * 
 *         if has_pct:             # <<<<<<<<<<<<<<
 *             pct[has_pct-1] = ch
 *             has_pct += 1
 */
    }

    /* "yarl/_quoting.pyx":129
 *             continue
 * 
 *         elif ch == '%':             # <<<<<<<<<<<<<<
 *             has_pct = 1
 * 
 */
    __pyx_t_3 = ((__pyx_v_ch == 37) != 0);
    if (__pyx_t_3) {

      /* "yarl/_quoting.pyx":130
 * 
 *         elif ch == '%':
 *             has_pct = 1             # <<<<<<<<<<<<<<
 * 
 *             # special case if "%" is last char
 */
      __pyx_v_has_pct = 1;

      /* "yarl/_quoting.pyx":133
 * 
 *             # special case if "%" is last char
 *             if idx == len(val):             # <<<<<<<<<<<<<<
 *                 if strict:
 *                     raise ValueError("Unallowed PCT %")
 */
      if (unlikely(__pyx_v_val == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
        __PYX_ERR(0, 133, __pyx_L1_error)
      }
      __pyx_t_1 = __Pyx_PyUnicode_GET_LENGTH(__pyx_v_val); if (unlikely(__pyx_t_1 == -1)) __PYX_ERR(0, 133, __pyx_L1_error)
      __pyx_t_3 = ((__pyx_v_idx == __pyx_t_1) != 0);
      if (__pyx_t_3) {

        /* "yarl/_quoting.pyx":134
 *             # special case if "%" is last char
 *             if idx == len(val):
 *                 if strict:             # <<<<<<<<<<<<<<
 *                     raise ValueError("Unallowed PCT %")
 *                 else:
 */
        __pyx_t_3 = (__pyx_v_strict != 0);
        if (__pyx_t_3) {

          /* "yarl/_quoting.pyx":135
 *             if idx == len(val):
 *                 if strict:
 *                     raise ValueError("Unallowed PCT %")             # <<<<<<<<<<<<<<
 *                 else:
 *                     PyUnicode_WriteChar(ret, ret_idx, '%')
 */
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 135, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_Raise(__pyx_t_2, 0, 0, 0);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __PYX_ERR(0, 135, __pyx_L1_error)

          /* "yarl/_quoting.pyx":134
 *             # special case if "%" is last char
 *             if idx == len(val):
 *                 if strict:             # <<<<<<<<<<<<<<
 *                     raise ValueError("Unallowed PCT %")
 *                 else:
 */
        }

        /* "yarl/_quoting.pyx":137
 *                     raise ValueError("Unallowed PCT %")
 *                 else:
 *                     PyUnicode_WriteChar(ret, ret_idx, '%')             # <<<<<<<<<<<<<<
 *                     ret_idx += 1
 *                     PyUnicode_WriteChar(ret, ret_idx, '2')
 */
        /*else*/ {
          PyUnicode_WriteChar(__pyx_v_ret, __pyx_v_ret_idx, 37);

          /* "yarl/_quoting.pyx":138
 *                 else:
 *                     PyUnicode_WriteChar(ret, ret_idx, '%')
 *                     ret_idx += 1             # <<<<<<<<<<<<<<
 *                     PyUnicode_WriteChar(ret, ret_idx, '2')
 *                     ret_idx += 1
 */
          __pyx_v_ret_idx = (__pyx_v_ret_idx + 1);

          /* "yarl/_quoting.pyx":139
 *                     PyUnicode_WriteChar(ret, ret_idx, '%')
 *                     ret_idx += 1
 *                     PyUnicode_WriteChar(ret, ret_idx, '2')             # <<<<<<<<<<<<<<
 *                     ret_idx += 1
 *                     PyUnicode_WriteChar(ret, ret_idx, '5')
 */
          PyUnicode_WriteChar(__pyx_v_ret, __pyx_v_ret_idx, 50);

          /* "yarl/_quoting.pyx":140
 *                     ret_idx += 1
 *                     PyUnicode_WriteChar(ret, ret_idx, '2')
 *                     ret_idx += 1             # <<<<<<<<<<<<<<
 *                     PyUnicode_WriteChar(ret, ret_idx, '5')
 *                     ret_idx += 1
 */
          __pyx_v_ret_idx = (__pyx_v_ret_idx + 1);

          /* "yarl/_quoting.pyx":141
 *                     PyUnicode_WriteChar(ret, ret_idx, '2')
 *                     ret_idx += 1
 *                     PyUnicode_WriteChar(ret, ret_idx, '5')             # <<<<<<<<<<<<<<
 *                     ret_idx += 1
 * 
 */
          PyUnicode_WriteChar(__pyx_v_ret, __pyx_v_ret_idx, 53);

          /* "yarl/_quoting.pyx":142
 *                     ret_idx += 1
 *                     PyUnicode_WriteChar(ret, ret_idx, '5')
 *                     ret_idx += 1             # <<<<<<<<<<<<<<
 * 
 *             continue
 */
          __pyx_v_ret_idx = (__pyx_v_ret_idx + 1);
        }

        /* "yarl/_quoting.pyx":133
 * 
 *             # special case if "%" is last char
 *             if idx == len(val):             # <<<<<<<<<<<<<<
 *                 if strict:
 *                     raise ValueError("Unallowed PCT %")
 */
      }

      /* "yarl/_quoting.pyx":144
 *                     ret_idx += 1
 * 
 *             continue             # <<<<<<<<<<<<<<
 * 
 *         if qs:
 */
      goto __pyx_L4_continue;

      /* "yarl/_quoting.pyx":129
 *             continue
 * 
 *         elif ch == '%':             # <<<<<<<<<<<<<<
 *             has_pct = 1
 * 
 */
    }

    /* "yarl/_quoting.pyx":146
 *             continue
 * 
 *         if qs:             # <<<<<<<<<<<<<<
 *             if ch == ' ':
 *                 PyUnicode_WriteChar(ret, ret_idx, '+')
 */
    __pyx_t_3 = (__pyx_v_qs != 0);
    if (__pyx_t_3) {

      /* "yarl/_quoting.pyx":147
 * 
 *         if qs:
 *             if ch == ' ':             # <<<<<<<<<<<<<<
 *                 PyUnicode_WriteChar(ret, ret_idx, '+')
 *                 ret_idx += 1
 */
      __pyx_t_3 = ((__pyx_v_ch == 32) != 0);
      if (__pyx_t_3) {

        /* "yarl/_quoting.pyx":148
 *         if qs:
 *             if ch == ' ':
 *                 PyUnicode_WriteChar(ret, ret_idx, '+')             # <<<<<<<<<<<<<<
 *                 ret_idx += 1
 *                 continue
 */
        PyUnicode_WriteChar(__pyx_v_ret, __pyx_v_ret_idx, 43);

        /* "yarl/_quoting.pyx":149
 *             if ch == ' ':
 *                 PyUnicode_WriteChar(ret, ret_idx, '+')
 *                 ret_idx += 1             # <<<<<<<<<<<<<<
 *                 continue
 *         if ch in safe:
 */
        __pyx_v_ret_idx = (__pyx_v_ret_idx + 1);

        /* "yarl/_quoting.pyx":150
 *                 PyUnicode_WriteChar(ret, ret_idx, '+')
 *                 ret_idx += 1
 *                 continue             # <<<<<<<<<<<<<<
 *         if ch in safe:
 *             PyUnicode_WriteChar(ret, ret_idx, ch)
 */
        goto __pyx_L4_continue;

        /* "yarl/_quoting.pyx":147
 * 
 *         if qs:
 *             if ch == ' ':             # <<<<<<<<<<<<<<
 *                 PyUnicode_WriteChar(ret, ret_idx, '+')
 *                 ret_idx += 1
 */
      }

      /* "yarl/_quoting.pyx":146
 *             continue
 * 
 *         if qs:             # <<<<<<<<<<<<<<
 *             if ch == ' ':
 *                 PyUnicode_WriteChar(ret, ret_idx, '+')
 */
    }

    /* "yarl/_quoting.pyx":151
 *                 ret_idx += 1
 *                 continue
 *         if ch in safe:             # <<<<<<<<<<<<<<
 *             PyUnicode_WriteChar(ret, ret_idx, ch)
 *             ret_idx +=1
 */
    __pyx_t_3 = ((__Pyx_UnicodeContainsUCS4(__pyx_v_safe, __pyx_v_ch)) != 0);
    if (__pyx_t_3) {

      /* "yarl/_quoting.pyx":152
 *                 continue
 *         if ch in safe:
 *             PyUnicode_WriteChar(ret, ret_idx, ch)             # <<<<<<<<<<<<<<
 *             ret_idx +=1
 *             continue
 */
      PyUnicode_WriteChar(__pyx_v_ret, __pyx_v_ret_idx, __pyx_v_ch);

      /* "yarl/_quoting.pyx":153
 *         if ch in safe:
 *             PyUnicode_WriteChar(ret, ret_idx, ch)
 *             ret_idx +=1             # <<<<<<<<<<<<<<
 *             continue
 *         if ch in ALLOWED:
 */
      __pyx_v_ret_idx = (__pyx_v_ret_idx + 1);

      /* "yarl/_quoting.pyx":154
 *             PyUnicode_WriteChar(ret, ret_idx, ch)
 *             ret_idx +=1
 *             continue             # <<<<<<<<<<<<<<
 *         if ch in ALLOWED:
 *             PyUnicode_WriteChar(ret, ret_idx, ch)
 */
      goto __pyx_L4_continue;

      /* "yarl/_quoting.pyx":151
 *                 ret_idx += 1
 *                 continue
 *         if ch in safe:             # <<<<<<<<<<<<<<
 *             PyUnicode_WriteChar(ret, ret_idx, ch)
 *             ret_idx +=1
 */
    }

    /* "yarl/_quoting.pyx":155
 *             ret_idx +=1
 *             continue
 *         if ch in ALLOWED:             # <<<<<<<<<<<<<<
 *             PyUnicode_WriteChar(ret, ret_idx, ch)
 *             ret_idx +=1
 */
    if (unlikely(__pyx_v_4yarl_8_quoting_ALLOWED == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "argument of type 'NoneType' is not iterable");
      __PYX_ERR(0, 155, __pyx_L1_error)
    }
    __pyx_t_3 = ((__Pyx_UnicodeContainsUCS4(__pyx_v_4yarl_8_quoting_ALLOWED, __pyx_v_ch)) != 0);
    if (__pyx_t_3) {

      /* "yarl/_quoting.pyx":156
 *             continue
 *         if ch in ALLOWED:
 *             PyUnicode_WriteChar(ret, ret_idx, ch)             # <<<<<<<<<<<<<<
 *             ret_idx +=1
 *             continue
 */
      PyUnicode_WriteChar(__pyx_v_ret, __pyx_v_ret_idx, __pyx_v_ch);

      /* "yarl/_quoting.pyx":157
 *         if ch in ALLOWED:
 *             PyUnicode_WriteChar(ret, ret_idx, ch)
 *             ret_idx +=1             # <<<<<<<<<<<<<<
 *             continue
 * 
 */
      __pyx_v_ret_idx = (__pyx_v_ret_idx + 1);

      /* "yarl/_quoting.pyx":158
 *             PyUnicode_WriteChar(ret, ret_idx, ch)
 *             ret_idx +=1
 *             continue             # <<<<<<<<<<<<<<
 * 
 *         ch_bytes = ch.encode("utf-8", errors= 'strict' if strict else 'ignore')
 */
      goto __pyx_L4_continue;

      /* "yarl/_quoting.pyx":155
 *             ret_idx +=1
 *             continue
 *         if ch in ALLOWED:             # <<<<<<<<<<<<<<
 *             PyUnicode_WriteChar(ret, ret_idx, ch)
 *             ret_idx +=1
 */
    }

    /* "yarl/_quoting.pyx":160
 *             continue
 * 
 *         ch_bytes = ch.encode("utf-8", errors= 'strict' if strict else 'ignore')             # <<<<<<<<<<<<<<
 * 
 *         for b in ch_bytes:
 */
    __pyx_t_2 = PyUnicode_FromOrdinal(__pyx_v_ch); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 160, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_encode); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 160, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 160, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if ((__pyx_v_strict != 0)) {
      __Pyx_INCREF(__pyx_n_u_strict);
      __pyx_t_7 = __pyx_n_u_strict;
    } else {
      __Pyx_INCREF(__pyx_n_u_ignore);
      __pyx_t_7 = __pyx_n_u_ignore;
    }
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_errors, __pyx_t_7) < 0) __PYX_ERR(0, 160, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_6, __pyx_tuple__5, __pyx_t_2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 160, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF_SET(__pyx_v_ch_bytes, __pyx_t_7);
    __pyx_t_7 = 0;

    /* "yarl/_quoting.pyx":162
 *         ch_bytes = ch.encode("utf-8", errors= 'strict' if strict else 'ignore')
 * 
 *         for b in ch_bytes:             # <<<<<<<<<<<<<<
 *             PyUnicode_WriteChar(ret, ret_idx, '%')
 *             ret_idx += 1
 */
    if (likely(PyList_CheckExact(__pyx_v_ch_bytes)) || PyTuple_CheckExact(__pyx_v_ch_bytes)) {
      __pyx_t_7 = __pyx_v_ch_bytes; __Pyx_INCREF(__pyx_t_7); __pyx_t_1 = 0;
      __pyx_t_12 = NULL;
    } else {
      __pyx_t_1 = -1; __pyx_t_7 = PyObject_GetIter(__pyx_v_ch_bytes); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 162, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_12 = Py_TYPE(__pyx_t_7)->tp_iternext; if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 162, __pyx_L1_error)
    }
    for (;;) {
      if (likely(!__pyx_t_12)) {
        if (likely(PyList_CheckExact(__pyx_t_7))) {
          if (__pyx_t_1 >= PyList_GET_SIZE(__pyx_t_7)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_2 = PyList_GET_ITEM(__pyx_t_7, __pyx_t_1); __Pyx_INCREF(__pyx_t_2); __pyx_t_1++; if (unlikely(0 < 0)) __PYX_ERR(0, 162, __pyx_L1_error)
          #else
          __pyx_t_2 = PySequence_ITEM(__pyx_t_7, __pyx_t_1); __pyx_t_1++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 162, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          #endif
        } else {
          if (__pyx_t_1 >= PyTuple_GET_SIZE(__pyx_t_7)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_2 = PyTuple_GET_ITEM(__pyx_t_7, __pyx_t_1); __Pyx_INCREF(__pyx_t_2); __pyx_t_1++; if (unlikely(0 < 0)) __PYX_ERR(0, 162, __pyx_L1_error)
          #else
          __pyx_t_2 = PySequence_ITEM(__pyx_t_7, __pyx_t_1); __pyx_t_1++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 162, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          #endif
        }
      } else {
        __pyx_t_2 = __pyx_t_12(__pyx_t_7);
        if (unlikely(!__pyx_t_2)) {
          PyObject* exc_type = PyErr_Occurred();
          if (exc_type) {
            if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
            else __PYX_ERR(0, 162, __pyx_L1_error)
          }
          break;
        }
        __Pyx_GOTREF(__pyx_t_2);
      }
      __pyx_t_13 = __Pyx_PyInt_As_uint8_t(__pyx_t_2); if (unlikely((__pyx_t_13 == ((uint8_t)-1)) && PyErr_Occurred())) __PYX_ERR(0, 162, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_v_b = __pyx_t_13;

      /* "yarl/_quoting.pyx":163
 * 
 *         for b in ch_bytes:
 *             PyUnicode_WriteChar(ret, ret_idx, '%')             # <<<<<<<<<<<<<<
 *             ret_idx += 1
 *             ch = _hex(<uint8_t>b >> 4)
 */
      PyUnicode_WriteChar(__pyx_v_ret, __pyx_v_ret_idx, 37);

      /* "yarl/_quoting.pyx":164
 *         for b in ch_bytes:
 *             PyUnicode_WriteChar(ret, ret_idx, '%')
 *             ret_idx += 1             # <<<<<<<<<<<<<<
 *             ch = _hex(<uint8_t>b >> 4)
 *             PyUnicode_WriteChar(ret, ret_idx, ch)
 */
      __pyx_v_ret_idx = (__pyx_v_ret_idx + 1);

      /* "yarl/_quoting.pyx":165
 *             PyUnicode_WriteChar(ret, ret_idx, '%')
 *             ret_idx += 1
 *             ch = _hex(<uint8_t>b >> 4)             # <<<<<<<<<<<<<<
 *             PyUnicode_WriteChar(ret, ret_idx, ch)
 *             ret_idx += 1
 */
      __pyx_v_ch = __pyx_f_4yarl_8_quoting__hex((((uint8_t)__pyx_v_b) >> 4));

      /* "yarl/_quoting.pyx":166
 *             ret_idx += 1
 *             ch = _hex(<uint8_t>b >> 4)
 *             PyUnicode_WriteChar(ret, ret_idx, ch)             # <<<<<<<<<<<<<<
 *             ret_idx += 1
 *             ch = _hex(<uint8_t>b & 0x0f)
 */
      PyUnicode_WriteChar(__pyx_v_ret, __pyx_v_ret_idx, __pyx_v_ch);

      /* "yarl/_quoting.pyx":167
 *             ch = _hex(<uint8_t>b >> 4)
 *             PyUnicode_WriteChar(ret, ret_idx, ch)
 *             ret_idx += 1             # <<<<<<<<<<<<<<
 *             ch = _hex(<uint8_t>b & 0x0f)
 *             PyUnicode_WriteChar(ret, ret_idx, ch)
 */
      __pyx_v_ret_idx = (__pyx_v_ret_idx + 1);

      /* "yarl/_quoting.pyx":168
 *             PyUnicode_WriteChar(ret, ret_idx, ch)
 *             ret_idx += 1
 *             ch = _hex(<uint8_t>b & 0x0f)             # <<<<<<<<<<<<<<
 *             PyUnicode_WriteChar(ret, ret_idx, ch)
 *             ret_idx += 1
 */
      __pyx_v_ch = __pyx_f_4yarl_8_quoting__hex((((uint8_t)__pyx_v_b) & 0x0f));

      /* "yarl/_quoting.pyx":169
 *             ret_idx += 1
 *             ch = _hex(<uint8_t>b & 0x0f)
 *             PyUnicode_WriteChar(ret, ret_idx, ch)             # <<<<<<<<<<<<<<
 *             ret_idx += 1
 * 
 */
      PyUnicode_WriteChar(__pyx_v_ret, __pyx_v_ret_idx, __pyx_v_ch);

      /* "yarl/_quoting.pyx":170
 *             ch = _hex(<uint8_t>b & 0x0f)
 *             PyUnicode_WriteChar(ret, ret_idx, ch)
 *             ret_idx += 1             # <<<<<<<<<<<<<<
 * 
 *     return PyUnicode_Substring(ret, 0, ret_idx)
 */
      __pyx_v_ret_idx = (__pyx_v_ret_idx + 1);

      /* "yarl/_quoting.pyx":162
 *         ch_bytes = ch.encode("utf-8", errors= 'strict' if strict else 'ignore')
 * 
 *         for b in ch_bytes:             # <<<<<<<<<<<<<<
 *             PyUnicode_WriteChar(ret, ret_idx, '%')
 *             ret_idx += 1
 */
    }
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_L4_continue:;
  }

  /* "yarl/_quoting.pyx":172
 *             ret_idx += 1
 * 
 *     return PyUnicode_Substring(ret, 0, ret_idx)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_7 = PyUnicode_Substring(__pyx_v_ret, 0, __pyx_v_ret_idx); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 172, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_7))||((__pyx_t_7) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_t_7)->tp_name), 0))) __PYX_ERR(0, 172, __pyx_L1_error)
  __pyx_r = ((PyObject*)__pyx_t_7);
  __pyx_t_7 = 0;
  goto __pyx_L0;

  /* "yarl/_quoting.pyx":48
 * 
 * 
 * cdef str _do_quote(str val, str safe, str protected, bint qs, bint strict):             # <<<<<<<<<<<<<<
 *     cdef uint8_t b
 *     cdef Py_UCS4 ch, unquoted
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("yarl._quoting._do_quote", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_ret);
  __Pyx_XDECREF(__pyx_v_ch_bytes);
  __Pyx_XDECREF(__pyx_v_safe);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "yarl/_quoting.pyx":175
 * 
 * 
 * def _unquote(val, *, unsafe='', qs=False):             # <<<<<<<<<<<<<<
 *     if val is None:
 *         return None
 */

/* Python wrapper */
static PyObject *__pyx_pw_4yarl_8_quoting_3_unquote(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_4yarl_8_quoting_3_unquote = {"_unquote", (PyCFunction)__pyx_pw_4yarl_8_quoting_3_unquote, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_4yarl_8_quoting_3_unquote(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_val = 0;
  PyObject *__pyx_v_unsafe = 0;
  PyObject *__pyx_v_qs = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_unquote (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_val,&__pyx_n_s_unsafe,&__pyx_n_s_qs,0};
    PyObject* values[3] = {0,0,0};
    values[1] = ((PyObject *)__pyx_kp_u_);
    values[2] = ((PyObject *)Py_False);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_val)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (kw_args > 0 && likely(kw_args <= 2)) {
        Py_ssize_t index;
        for (index = 1; index < 3 && kw_args > 0; index++) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, *__pyx_pyargnames[index]);
          if (value) { values[index] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_unquote") < 0)) __PYX_ERR(0, 175, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_val = values[0];
    __pyx_v_unsafe = values[1];
    __pyx_v_qs = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_unquote", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 175, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("yarl._quoting._unquote", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_4yarl_8_quoting_2_unquote(__pyx_self, __pyx_v_val, __pyx_v_unsafe, __pyx_v_qs);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_4yarl_8_quoting_2_unquote(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_val, PyObject *__pyx_v_unsafe, PyObject *__pyx_v_qs) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  struct __pyx_opt_args_4yarl_8_quoting__do_unquote __pyx_t_4;
  __Pyx_RefNannySetupContext("_unquote", 0);

  /* "yarl/_quoting.pyx":176
 * 
 * def _unquote(val, *, unsafe='', qs=False):
 *     if val is None:             # <<<<<<<<<<<<<<
 *         return None
 *     if not isinstance(val, str):
 */
  __pyx_t_1 = (__pyx_v_val == Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "yarl/_quoting.pyx":177
 * def _unquote(val, *, unsafe='', qs=False):
 *     if val is None:
 *         return None             # <<<<<<<<<<<<<<
 *     if not isinstance(val, str):
 *         raise TypeError("Argument should be str")
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_None);
    __pyx_r = Py_None;
    goto __pyx_L0;

    /* "yarl/_quoting.pyx":176
 * 
 * def _unquote(val, *, unsafe='', qs=False):
 *     if val is None:             # <<<<<<<<<<<<<<
 *         return None
 *     if not isinstance(val, str):
 */
  }

  /* "yarl/_quoting.pyx":178
 *     if val is None:
 *         return None
 *     if not isinstance(val, str):             # <<<<<<<<<<<<<<
 *         raise TypeError("Argument should be str")
 *     if not val:
 */
  __pyx_t_2 = PyUnicode_Check(__pyx_v_val); 
  __pyx_t_1 = ((!(__pyx_t_2 != 0)) != 0);
  if (__pyx_t_1) {

    /* "yarl/_quoting.pyx":179
 *         return None
 *     if not isinstance(val, str):
 *         raise TypeError("Argument should be str")             # <<<<<<<<<<<<<<
 *     if not val:
 *         return ''
 */
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__6, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 179, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 179, __pyx_L1_error)

    /* "yarl/_quoting.pyx":178
 *     if val is None:
 *         return None
 *     if not isinstance(val, str):             # <<<<<<<<<<<<<<
 *         raise TypeError("Argument should be str")
 *     if not val:
 */
  }

  /* "yarl/_quoting.pyx":180
 *     if not isinstance(val, str):
 *         raise TypeError("Argument should be str")
 *     if not val:             # <<<<<<<<<<<<<<
 *         return ''
 *     return _do_unquote(<str>val, unsafe, qs)
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_val); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 180, __pyx_L1_error)
  __pyx_t_2 = ((!__pyx_t_1) != 0);
  if (__pyx_t_2) {

    /* "yarl/_quoting.pyx":181
 *         raise TypeError("Argument should be str")
 *     if not val:
 *         return ''             # <<<<<<<<<<<<<<
 *     return _do_unquote(<str>val, unsafe, qs)
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_kp_u_);
    __pyx_r = __pyx_kp_u_;
    goto __pyx_L0;

    /* "yarl/_quoting.pyx":180
 *     if not isinstance(val, str):
 *         raise TypeError("Argument should be str")
 *     if not val:             # <<<<<<<<<<<<<<
 *         return ''
 *     return _do_unquote(<str>val, unsafe, qs)
 */
  }

  /* "yarl/_quoting.pyx":182
 *     if not val:
 *         return ''
 *     return _do_unquote(<str>val, unsafe, qs)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  if (!(likely(PyUnicode_CheckExact(__pyx_v_unsafe))||((__pyx_v_unsafe) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_v_unsafe)->tp_name), 0))) __PYX_ERR(0, 182, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_qs); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 182, __pyx_L1_error)
  __pyx_t_4.__pyx_n = 2;
  __pyx_t_4.unsafe = ((PyObject*)__pyx_v_unsafe);
  __pyx_t_4.qs = __pyx_t_2;
  __pyx_t_3 = __pyx_f_4yarl_8_quoting__do_unquote(((PyObject*)__pyx_v_val), &__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 182, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "yarl/_quoting.pyx":175
 * 
 * 
 * def _unquote(val, *, unsafe='', qs=False):             # <<<<<<<<<<<<<<
 *     if val is None:
 *         return None
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("yarl._quoting._unquote", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "yarl/_quoting.pyx":185
 * 
 * 
 * cdef str _do_unquote(str val, str unsafe='', bint qs=False):             # <<<<<<<<<<<<<<
 *     cdef str pct = ''
 *     cdef str last_pct = ''
 */

static PyObject *__pyx_f_4yarl_8_quoting__do_unquote(PyObject *__pyx_v_val, struct __pyx_opt_args_4yarl_8_quoting__do_unquote *__pyx_optional_args) {
  PyObject *__pyx_v_unsafe = ((PyObject*)__pyx_kp_u_);
  int __pyx_v_qs = ((int)0);
  PyObject *__pyx_v_pct = 0;
  PyObject *__pyx_v_last_pct = 0;
  PyObject *__pyx_v_pcts = 0;
  PyObject *__pyx_v_ret = 0;
  PyObject *__pyx_v_unquoted = 0;
  PyObject *__pyx_v_ch = NULL;
  PyObject *__pyx_v_h = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  void *__pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  int __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  int __pyx_t_13;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  int __pyx_t_17;
  int __pyx_t_18;
  long __pyx_t_19;
  PyObject *(*__pyx_t_20)(PyObject *);
  __Pyx_RefNannySetupContext("_do_unquote", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_unsafe = __pyx_optional_args->unsafe;
      if (__pyx_optional_args->__pyx_n > 1) {
        __pyx_v_qs = __pyx_optional_args->qs;
      }
    }
  }

  /* "yarl/_quoting.pyx":186
 * 
 * cdef str _do_unquote(str val, str unsafe='', bint qs=False):
 *     cdef str pct = ''             # <<<<<<<<<<<<<<
 *     cdef str last_pct = ''
 *     cdef bytearray pcts = bytearray()
 */
  __Pyx_INCREF(__pyx_kp_u_);
  __pyx_v_pct = __pyx_kp_u_;

  /* "yarl/_quoting.pyx":187
 * cdef str _do_unquote(str val, str unsafe='', bint qs=False):
 *     cdef str pct = ''
 *     cdef str last_pct = ''             # <<<<<<<<<<<<<<
 *     cdef bytearray pcts = bytearray()
 *     cdef list ret = []
 */
  __Pyx_INCREF(__pyx_kp_u_);
  __pyx_v_last_pct = __pyx_kp_u_;

  /* "yarl/_quoting.pyx":188
 *     cdef str pct = ''
 *     cdef str last_pct = ''
 *     cdef bytearray pcts = bytearray()             # <<<<<<<<<<<<<<
 *     cdef list ret = []
 *     cdef str unquoted
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)(&PyByteArray_Type)), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 188, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_pcts = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "yarl/_quoting.pyx":189
 *     cdef str last_pct = ''
 *     cdef bytearray pcts = bytearray()
 *     cdef list ret = []             # <<<<<<<<<<<<<<
 *     cdef str unquoted
 *     for ch in val:
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 189, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_ret = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "yarl/_quoting.pyx":191
 *     cdef list ret = []
 *     cdef str unquoted
 *     for ch in val:             # <<<<<<<<<<<<<<
 *         if pct:
 *             pct += ch
 */
  if (unlikely(__pyx_v_val == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' is not iterable");
    __PYX_ERR(0, 191, __pyx_L1_error)
  }
  __Pyx_INCREF(__pyx_v_val);
  __pyx_t_2 = __pyx_v_val;
  __pyx_t_7 = __Pyx_init_unicode_iteration(__pyx_t_2, (&__pyx_t_4), (&__pyx_t_5), (&__pyx_t_6)); if (unlikely(__pyx_t_7 == -1)) __PYX_ERR(0, 191, __pyx_L1_error)
  for (__pyx_t_8 = 0; __pyx_t_8 < __pyx_t_4; __pyx_t_8++) {
    __pyx_t_3 = __pyx_t_8;
    __pyx_t_1 = PyUnicode_FromOrdinal(__Pyx_PyUnicode_READ(__pyx_t_6, __pyx_t_5, __pyx_t_3)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 191, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_ch, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "yarl/_quoting.pyx":192
 *     cdef str unquoted
 *     for ch in val:
 *         if pct:             # <<<<<<<<<<<<<<
 *             pct += ch
 *             if len(pct) == 3:  # pragma: no branch   # peephole optimizer
 */
    __pyx_t_9 = (__pyx_v_pct != Py_None) && (__Pyx_PyUnicode_IS_TRUE(__pyx_v_pct) != 0);
    if (__pyx_t_9) {

      /* "yarl/_quoting.pyx":193
 *     for ch in val:
 *         if pct:
 *             pct += ch             # <<<<<<<<<<<<<<
 *             if len(pct) == 3:  # pragma: no branch   # peephole optimizer
 *                 pcts.append(int(pct[1:], base=16))
 */
      __pyx_t_1 = PyNumber_InPlaceAdd(__pyx_v_pct, __pyx_v_ch); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 193, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 193, __pyx_L1_error)
      __Pyx_DECREF_SET(__pyx_v_pct, ((PyObject*)__pyx_t_1));
      __pyx_t_1 = 0;

      /* "yarl/_quoting.pyx":194
 *         if pct:
 *             pct += ch
 *             if len(pct) == 3:  # pragma: no branch   # peephole optimizer             # <<<<<<<<<<<<<<
 *                 pcts.append(int(pct[1:], base=16))
 *                 last_pct = pct
 */
      if (unlikely(__pyx_v_pct == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
        __PYX_ERR(0, 194, __pyx_L1_error)
      }
      __pyx_t_10 = __Pyx_PyUnicode_GET_LENGTH(__pyx_v_pct); if (unlikely(__pyx_t_10 == -1)) __PYX_ERR(0, 194, __pyx_L1_error)
      __pyx_t_9 = ((__pyx_t_10 == 3) != 0);
      if (__pyx_t_9) {

        /* "yarl/_quoting.pyx":195
 *             pct += ch
 *             if len(pct) == 3:  # pragma: no branch   # peephole optimizer
 *                 pcts.append(int(pct[1:], base=16))             # <<<<<<<<<<<<<<
 *                 last_pct = pct
 *                 pct = ''
 */
        if (unlikely(__pyx_v_pct == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 195, __pyx_L1_error)
        }
        __pyx_t_1 = __Pyx_PyUnicode_Substring(__pyx_v_pct, 1, PY_SSIZE_T_MAX); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 195, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_11 = PyTuple_New(1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 195, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_GIVEREF(__pyx_t_1);
        PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_1);
        __pyx_t_1 = 0;
        __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 195, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        if (PyDict_SetItem(__pyx_t_1, __pyx_n_s_base, __pyx_int_16) < 0) __PYX_ERR(0, 195, __pyx_L1_error)
        __pyx_t_12 = __Pyx_PyObject_Call(((PyObject *)(&PyInt_Type)), __pyx_t_11, __pyx_t_1); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 195, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_12);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_13 = __Pyx_PyByteArray_AppendObject(__pyx_v_pcts, __pyx_t_12); if (unlikely(__pyx_t_13 == -1)) __PYX_ERR(0, 195, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

        /* "yarl/_quoting.pyx":196
 *             if len(pct) == 3:  # pragma: no branch   # peephole optimizer
 *                 pcts.append(int(pct[1:], base=16))
 *                 last_pct = pct             # <<<<<<<<<<<<<<
 *                 pct = ''
 *             continue
 */
        __Pyx_INCREF(__pyx_v_pct);
        __Pyx_DECREF_SET(__pyx_v_last_pct, __pyx_v_pct);

        /* "yarl/_quoting.pyx":197
 *                 pcts.append(int(pct[1:], base=16))
 *                 last_pct = pct
 *                 pct = ''             # <<<<<<<<<<<<<<
 *             continue
 *         if pcts:
 */
        __Pyx_INCREF(__pyx_kp_u_);
        __Pyx_DECREF_SET(__pyx_v_pct, __pyx_kp_u_);

        /* "yarl/_quoting.pyx":194
 *         if pct:
 *             pct += ch
 *             if len(pct) == 3:  # pragma: no branch   # peephole optimizer             # <<<<<<<<<<<<<<
 *                 pcts.append(int(pct[1:], base=16))
 *                 last_pct = pct
 */
      }

      /* "yarl/_quoting.pyx":198
 *                 last_pct = pct
 *                 pct = ''
 *             continue             # <<<<<<<<<<<<<<
 *         if pcts:
 *             try:
 */
      goto __pyx_L3_continue;

      /* "yarl/_quoting.pyx":192
 *     cdef str unquoted
 *     for ch in val:
 *         if pct:             # <<<<<<<<<<<<<<
 *             pct += ch
 *             if len(pct) == 3:  # pragma: no branch   # peephole optimizer
 */
    }

    /* "yarl/_quoting.pyx":199
 *                 pct = ''
 *             continue
 *         if pcts:             # <<<<<<<<<<<<<<
 *             try:
 *                 unquoted = pcts.decode('utf8')
 */
    __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_v_pcts); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 199, __pyx_L1_error)
    if (__pyx_t_9) {

      /* "yarl/_quoting.pyx":200
 *             continue
 *         if pcts:
 *             try:             # <<<<<<<<<<<<<<
 *                 unquoted = pcts.decode('utf8')
 *             except UnicodeDecodeError:
 */
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_14, &__pyx_t_15, &__pyx_t_16);
        __Pyx_XGOTREF(__pyx_t_14);
        __Pyx_XGOTREF(__pyx_t_15);
        __Pyx_XGOTREF(__pyx_t_16);
        /*try:*/ {

          /* "yarl/_quoting.pyx":201
 *         if pcts:
 *             try:
 *                 unquoted = pcts.decode('utf8')             # <<<<<<<<<<<<<<
 *             except UnicodeDecodeError:
 *                 pass
 */
          __pyx_t_12 = __Pyx_decode_bytearray(__pyx_v_pcts, 0, PY_SSIZE_T_MAX, NULL, NULL, PyUnicode_DecodeUTF8); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 201, __pyx_L8_error)
          __Pyx_GOTREF(__pyx_t_12);
          if (!(likely(PyUnicode_CheckExact(__pyx_t_12))||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_t_12)->tp_name), 0))) __PYX_ERR(0, 201, __pyx_L8_error)
          __Pyx_XDECREF_SET(__pyx_v_unquoted, ((PyObject*)__pyx_t_12));
          __pyx_t_12 = 0;

          /* "yarl/_quoting.pyx":200
 *             continue
 *         if pcts:
 *             try:             # <<<<<<<<<<<<<<
 *                 unquoted = pcts.decode('utf8')
 *             except UnicodeDecodeError:
 */
        }

        /* "yarl/_quoting.pyx":205
 *                 pass
 *             else:
 *                 if qs and unquoted in '+=&;':             # <<<<<<<<<<<<<<
 *                     ret.append(_do_quote(unquoted, '', '', True, True))
 *                 elif unquoted in unsafe:
 */
        /*else:*/ {
          __pyx_t_17 = (__pyx_v_qs != 0);
          if (__pyx_t_17) {
          } else {
            __pyx_t_9 = __pyx_t_17;
            goto __pyx_L17_bool_binop_done;
          }
          __pyx_t_17 = (__Pyx_PyUnicode_ContainsTF(__pyx_v_unquoted, __pyx_kp_u__7, Py_EQ)); if (unlikely(__pyx_t_17 < 0)) __PYX_ERR(0, 205, __pyx_L10_except_error)
          __pyx_t_18 = (__pyx_t_17 != 0);
          __pyx_t_9 = __pyx_t_18;
          __pyx_L17_bool_binop_done:;
          if (__pyx_t_9) {

            /* "yarl/_quoting.pyx":206
 *             else:
 *                 if qs and unquoted in '+=&;':
 *                     ret.append(_do_quote(unquoted, '', '', True, True))             # <<<<<<<<<<<<<<
 *                 elif unquoted in unsafe:
 *                     ret.append(_do_quote(unquoted, '', '', False, True))
 */
            __pyx_t_12 = __pyx_f_4yarl_8_quoting__do_quote(__pyx_v_unquoted, __pyx_kp_u_, __pyx_kp_u_, 1, 1); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 206, __pyx_L10_except_error)
            __Pyx_GOTREF(__pyx_t_12);
            __pyx_t_13 = __Pyx_PyList_Append(__pyx_v_ret, __pyx_t_12); if (unlikely(__pyx_t_13 == -1)) __PYX_ERR(0, 206, __pyx_L10_except_error)
            __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

            /* "yarl/_quoting.pyx":205
 *                 pass
 *             else:
 *                 if qs and unquoted in '+=&;':             # <<<<<<<<<<<<<<
 *                     ret.append(_do_quote(unquoted, '', '', True, True))
 *                 elif unquoted in unsafe:
 */
            goto __pyx_L16;
          }

          /* "yarl/_quoting.pyx":207
 *                 if qs and unquoted in '+=&;':
 *                     ret.append(_do_quote(unquoted, '', '', True, True))
 *                 elif unquoted in unsafe:             # <<<<<<<<<<<<<<
 *                     ret.append(_do_quote(unquoted, '', '', False, True))
 *                 else:
 */
          if (unlikely(__pyx_v_unsafe == Py_None)) {
            PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
            __PYX_ERR(0, 207, __pyx_L10_except_error)
          }
          __pyx_t_9 = (__Pyx_PyUnicode_ContainsTF(__pyx_v_unquoted, __pyx_v_unsafe, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 207, __pyx_L10_except_error)
          __pyx_t_18 = (__pyx_t_9 != 0);
          if (__pyx_t_18) {

            /* "yarl/_quoting.pyx":208
 *                     ret.append(_do_quote(unquoted, '', '', True, True))
 *                 elif unquoted in unsafe:
 *                     ret.append(_do_quote(unquoted, '', '', False, True))             # <<<<<<<<<<<<<<
 *                 else:
 *                     ret.append(unquoted)
 */
            __pyx_t_12 = __pyx_f_4yarl_8_quoting__do_quote(__pyx_v_unquoted, __pyx_kp_u_, __pyx_kp_u_, 0, 1); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 208, __pyx_L10_except_error)
            __Pyx_GOTREF(__pyx_t_12);
            __pyx_t_13 = __Pyx_PyList_Append(__pyx_v_ret, __pyx_t_12); if (unlikely(__pyx_t_13 == -1)) __PYX_ERR(0, 208, __pyx_L10_except_error)
            __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

            /* "yarl/_quoting.pyx":207
 *                 if qs and unquoted in '+=&;':
 *                     ret.append(_do_quote(unquoted, '', '', True, True))
 *                 elif unquoted in unsafe:             # <<<<<<<<<<<<<<
 *                     ret.append(_do_quote(unquoted, '', '', False, True))
 *                 else:
 */
            goto __pyx_L16;
          }

          /* "yarl/_quoting.pyx":210
 *                     ret.append(_do_quote(unquoted, '', '', False, True))
 *                 else:
 *                     ret.append(unquoted)             # <<<<<<<<<<<<<<
 *                 del pcts[:]
 * 
 */
          /*else*/ {
            __pyx_t_13 = __Pyx_PyList_Append(__pyx_v_ret, __pyx_v_unquoted); if (unlikely(__pyx_t_13 == -1)) __PYX_ERR(0, 210, __pyx_L10_except_error)
          }
          __pyx_L16:;

          /* "yarl/_quoting.pyx":211
 *                 else:
 *                     ret.append(unquoted)
 *                 del pcts[:]             # <<<<<<<<<<<<<<
 * 
 *         if ch == '%':
 */
          if (__Pyx_PyObject_DelSlice(__pyx_v_pcts, 0, 0, NULL, NULL, NULL, 0, 0, 1) < 0) __PYX_ERR(0, 211, __pyx_L10_except_error)
        }
        __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
        __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
        goto __pyx_L15_try_end;
        __pyx_L8_error:;
        __Pyx_PyThreadState_assign
        __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;

        /* "yarl/_quoting.pyx":202
 *             try:
 *                 unquoted = pcts.decode('utf8')
 *             except UnicodeDecodeError:             # <<<<<<<<<<<<<<
 *                 pass
 *             else:
 */
        __pyx_t_7 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_UnicodeDecodeError);
        if (__pyx_t_7) {
          __Pyx_ErrRestore(0,0,0);
          goto __pyx_L9_exception_handled;
        }
        goto __pyx_L10_except_error;
        __pyx_L10_except_error:;

        /* "yarl/_quoting.pyx":200
 *             continue
 *         if pcts:
 *             try:             # <<<<<<<<<<<<<<
 *                 unquoted = pcts.decode('utf8')
 *             except UnicodeDecodeError:
 */
        __Pyx_PyThreadState_assign
        __Pyx_XGIVEREF(__pyx_t_14);
        __Pyx_XGIVEREF(__pyx_t_15);
        __Pyx_XGIVEREF(__pyx_t_16);
        __Pyx_ExceptionReset(__pyx_t_14, __pyx_t_15, __pyx_t_16);
        goto __pyx_L1_error;
        __pyx_L9_exception_handled:;
        __Pyx_PyThreadState_assign
        __Pyx_XGIVEREF(__pyx_t_14);
        __Pyx_XGIVEREF(__pyx_t_15);
        __Pyx_XGIVEREF(__pyx_t_16);
        __Pyx_ExceptionReset(__pyx_t_14, __pyx_t_15, __pyx_t_16);
        __pyx_L15_try_end:;
      }

      /* "yarl/_quoting.pyx":199
 *                 pct = ''
 *             continue
 *         if pcts:             # <<<<<<<<<<<<<<
 *             try:
 *                 unquoted = pcts.decode('utf8')
 */
    }

    /* "yarl/_quoting.pyx":213
 *                 del pcts[:]
 * 
 *         if ch == '%':             # <<<<<<<<<<<<<<
 *             pct = ch
 *             continue
 */
    __pyx_t_18 = (__Pyx_PyUnicode_Equals(__pyx_v_ch, __pyx_kp_u__8, Py_EQ)); if (unlikely(__pyx_t_18 < 0)) __PYX_ERR(0, 213, __pyx_L1_error)
    if (__pyx_t_18) {

      /* "yarl/_quoting.pyx":214
 * 
 *         if ch == '%':
 *             pct = ch             # <<<<<<<<<<<<<<
 *             continue
 * 
 */
      if (!(likely(PyUnicode_CheckExact(__pyx_v_ch))||((__pyx_v_ch) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_v_ch)->tp_name), 0))) __PYX_ERR(0, 214, __pyx_L1_error)
      __pyx_t_12 = __pyx_v_ch;
      __Pyx_INCREF(__pyx_t_12);
      __Pyx_DECREF_SET(__pyx_v_pct, ((PyObject*)__pyx_t_12));
      __pyx_t_12 = 0;

      /* "yarl/_quoting.pyx":215
 *         if ch == '%':
 *             pct = ch
 *             continue             # <<<<<<<<<<<<<<
 * 
 *         if pcts:
 */
      goto __pyx_L3_continue;

      /* "yarl/_quoting.pyx":213
 *                 del pcts[:]
 * 
 *         if ch == '%':             # <<<<<<<<<<<<<<
 *             pct = ch
 *             continue
 */
    }

    /* "yarl/_quoting.pyx":217
 *             continue
 * 
 *         if pcts:             # <<<<<<<<<<<<<<
 *             ret.append(last_pct)  # %F8ab
 *             last_pct = ''
 */
    __pyx_t_18 = __Pyx_PyObject_IsTrue(__pyx_v_pcts); if (unlikely(__pyx_t_18 < 0)) __PYX_ERR(0, 217, __pyx_L1_error)
    if (__pyx_t_18) {

      /* "yarl/_quoting.pyx":218
 * 
 *         if pcts:
 *             ret.append(last_pct)  # %F8ab             # <<<<<<<<<<<<<<
 *             last_pct = ''
 * 
 */
      __pyx_t_13 = __Pyx_PyList_Append(__pyx_v_ret, __pyx_v_last_pct); if (unlikely(__pyx_t_13 == -1)) __PYX_ERR(0, 218, __pyx_L1_error)

      /* "yarl/_quoting.pyx":219
 *         if pcts:
 *             ret.append(last_pct)  # %F8ab
 *             last_pct = ''             # <<<<<<<<<<<<<<
 * 
 *         if ch == '+':
 */
      __Pyx_INCREF(__pyx_kp_u_);
      __Pyx_DECREF_SET(__pyx_v_last_pct, __pyx_kp_u_);

      /* "yarl/_quoting.pyx":217
 *             continue
 * 
 *         if pcts:             # <<<<<<<<<<<<<<
 *             ret.append(last_pct)  # %F8ab
 *             last_pct = ''
 */
    }

    /* "yarl/_quoting.pyx":221
 *             last_pct = ''
 * 
 *         if ch == '+':             # <<<<<<<<<<<<<<
 *             if ch in unsafe:
 *                 ret.append('+')
 */
    __pyx_t_18 = (__Pyx_PyUnicode_Equals(__pyx_v_ch, __pyx_kp_u__9, Py_EQ)); if (unlikely(__pyx_t_18 < 0)) __PYX_ERR(0, 221, __pyx_L1_error)
    if (__pyx_t_18) {

      /* "yarl/_quoting.pyx":222
 * 
 *         if ch == '+':
 *             if ch in unsafe:             # <<<<<<<<<<<<<<
 *                 ret.append('+')
 *             else:
 */
      if (unlikely(__pyx_v_unsafe == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
        __PYX_ERR(0, 222, __pyx_L1_error)
      }
      __pyx_t_18 = (__Pyx_PyUnicode_ContainsTF(__pyx_v_ch, __pyx_v_unsafe, Py_EQ)); if (unlikely(__pyx_t_18 < 0)) __PYX_ERR(0, 222, __pyx_L1_error)
      __pyx_t_9 = (__pyx_t_18 != 0);
      if (__pyx_t_9) {

        /* "yarl/_quoting.pyx":223
 *         if ch == '+':
 *             if ch in unsafe:
 *                 ret.append('+')             # <<<<<<<<<<<<<<
 *             else:
 *                 ret.append(' ')
 */
        __pyx_t_13 = __Pyx_PyList_Append(__pyx_v_ret, __pyx_kp_u__9); if (unlikely(__pyx_t_13 == -1)) __PYX_ERR(0, 223, __pyx_L1_error)

        /* "yarl/_quoting.pyx":222
 * 
 *         if ch == '+':
 *             if ch in unsafe:             # <<<<<<<<<<<<<<
 *                 ret.append('+')
 *             else:
 */
        goto __pyx_L22;
      }

      /* "yarl/_quoting.pyx":225
 *                 ret.append('+')
 *             else:
 *                 ret.append(' ')             # <<<<<<<<<<<<<<
 *             continue
 * 
 */
      /*else*/ {
        __pyx_t_13 = __Pyx_PyList_Append(__pyx_v_ret, __pyx_kp_u__10); if (unlikely(__pyx_t_13 == -1)) __PYX_ERR(0, 225, __pyx_L1_error)
      }
      __pyx_L22:;

      /* "yarl/_quoting.pyx":226
 *             else:
 *                 ret.append(' ')
 *             continue             # <<<<<<<<<<<<<<
 * 
 *         if ch in unsafe:
 */
      goto __pyx_L3_continue;

      /* "yarl/_quoting.pyx":221
 *             last_pct = ''
 * 
 *         if ch == '+':             # <<<<<<<<<<<<<<
 *             if ch in unsafe:
 *                 ret.append('+')
 */
    }

    /* "yarl/_quoting.pyx":228
 *             continue
 * 
 *         if ch in unsafe:             # <<<<<<<<<<<<<<
 *             ret.append('%')
 *             h = hex(ord(ch)).upper()[2:]
 */
    if (unlikely(__pyx_v_unsafe == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
      __PYX_ERR(0, 228, __pyx_L1_error)
    }
    __pyx_t_9 = (__Pyx_PyUnicode_ContainsTF(__pyx_v_ch, __pyx_v_unsafe, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 228, __pyx_L1_error)
    __pyx_t_18 = (__pyx_t_9 != 0);
    if (__pyx_t_18) {

      /* "yarl/_quoting.pyx":229
 * 
 *         if ch in unsafe:
 *             ret.append('%')             # <<<<<<<<<<<<<<
 *             h = hex(ord(ch)).upper()[2:]
 *             for ch in h:
 */
      __pyx_t_13 = __Pyx_PyList_Append(__pyx_v_ret, __pyx_kp_u__8); if (unlikely(__pyx_t_13 == -1)) __PYX_ERR(0, 229, __pyx_L1_error)

      /* "yarl/_quoting.pyx":230
 *         if ch in unsafe:
 *             ret.append('%')
 *             h = hex(ord(ch)).upper()[2:]             # <<<<<<<<<<<<<<
 *             for ch in h:
 *                 ret.append(ch)
 */
      __pyx_t_19 = __Pyx_PyObject_Ord(__pyx_v_ch); if (unlikely(__pyx_t_19 == (long)(Py_UCS4)-1)) __PYX_ERR(0, 230, __pyx_L1_error)
      __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_t_19); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 230, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_11 = PyTuple_New(1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 230, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_GIVEREF(__pyx_t_1);
      PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_1);
      __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_hex, __pyx_t_11, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 230, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_upper); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 230, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_11))) {
        __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_11);
        if (likely(__pyx_t_1)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_11);
          __Pyx_INCREF(__pyx_t_1);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_11, function);
        }
      }
      if (__pyx_t_1) {
        __pyx_t_12 = __Pyx_PyObject_CallOneArg(__pyx_t_11, __pyx_t_1); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 230, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      } else {
        __pyx_t_12 = __Pyx_PyObject_CallNoArg(__pyx_t_11); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 230, __pyx_L1_error)
      }
      __Pyx_GOTREF(__pyx_t_12);
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __pyx_t_11 = __Pyx_PyObject_GetSlice(__pyx_t_12, 2, 0, NULL, NULL, &__pyx_slice__11, 1, 0, 1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 230, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __Pyx_XDECREF_SET(__pyx_v_h, __pyx_t_11);
      __pyx_t_11 = 0;

      /* "yarl/_quoting.pyx":231
 *             ret.append('%')
 *             h = hex(ord(ch)).upper()[2:]
 *             for ch in h:             # <<<<<<<<<<<<<<
 *                 ret.append(ch)
 *             continue
 */
      if (likely(PyList_CheckExact(__pyx_v_h)) || PyTuple_CheckExact(__pyx_v_h)) {
        __pyx_t_11 = __pyx_v_h; __Pyx_INCREF(__pyx_t_11); __pyx_t_10 = 0;
        __pyx_t_20 = NULL;
      } else {
        __pyx_t_10 = -1; __pyx_t_11 = PyObject_GetIter(__pyx_v_h); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 231, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __pyx_t_20 = Py_TYPE(__pyx_t_11)->tp_iternext; if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 231, __pyx_L1_error)
      }
      for (;;) {
        if (likely(!__pyx_t_20)) {
          if (likely(PyList_CheckExact(__pyx_t_11))) {
            if (__pyx_t_10 >= PyList_GET_SIZE(__pyx_t_11)) break;
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_12 = PyList_GET_ITEM(__pyx_t_11, __pyx_t_10); __Pyx_INCREF(__pyx_t_12); __pyx_t_10++; if (unlikely(0 < 0)) __PYX_ERR(0, 231, __pyx_L1_error)
            #else
            __pyx_t_12 = PySequence_ITEM(__pyx_t_11, __pyx_t_10); __pyx_t_10++; if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 231, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_12);
            #endif
          } else {
            if (__pyx_t_10 >= PyTuple_GET_SIZE(__pyx_t_11)) break;
            #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
            __pyx_t_12 = PyTuple_GET_ITEM(__pyx_t_11, __pyx_t_10); __Pyx_INCREF(__pyx_t_12); __pyx_t_10++; if (unlikely(0 < 0)) __PYX_ERR(0, 231, __pyx_L1_error)
            #else
            __pyx_t_12 = PySequence_ITEM(__pyx_t_11, __pyx_t_10); __pyx_t_10++; if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 231, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_12);
            #endif
          }
        } else {
          __pyx_t_12 = __pyx_t_20(__pyx_t_11);
          if (unlikely(!__pyx_t_12)) {
            PyObject* exc_type = PyErr_Occurred();
            if (exc_type) {
              if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
              else __PYX_ERR(0, 231, __pyx_L1_error)
            }
            break;
          }
          __Pyx_GOTREF(__pyx_t_12);
        }
        __Pyx_DECREF_SET(__pyx_v_ch, __pyx_t_12);
        __pyx_t_12 = 0;

        /* "yarl/_quoting.pyx":232
 *             h = hex(ord(ch)).upper()[2:]
 *             for ch in h:
 *                 ret.append(ch)             # <<<<<<<<<<<<<<
 *             continue
 * 
 */
        __pyx_t_13 = __Pyx_PyList_Append(__pyx_v_ret, __pyx_v_ch); if (unlikely(__pyx_t_13 == -1)) __PYX_ERR(0, 232, __pyx_L1_error)

        /* "yarl/_quoting.pyx":231
 *             ret.append('%')
 *             h = hex(ord(ch)).upper()[2:]
 *             for ch in h:             # <<<<<<<<<<<<<<
 *                 ret.append(ch)
 *             continue
 */
      }
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

      /* "yarl/_quoting.pyx":233
 *             for ch in h:
 *                 ret.append(ch)
 *             continue             # <<<<<<<<<<<<<<
 * 
 *         ret.append(ch)
 */
      goto __pyx_L3_continue;

      /* "yarl/_quoting.pyx":228
 *             continue
 * 
 *         if ch in unsafe:             # <<<<<<<<<<<<<<
 *             ret.append('%')
 *             h = hex(ord(ch)).upper()[2:]
 */
    }

    /* "yarl/_quoting.pyx":235
 *             continue
 * 
 *         ret.append(ch)             # <<<<<<<<<<<<<<
 * 
 *     if pcts:
 */
    __pyx_t_13 = __Pyx_PyList_Append(__pyx_v_ret, __pyx_v_ch); if (unlikely(__pyx_t_13 == -1)) __PYX_ERR(0, 235, __pyx_L1_error)
    __pyx_L3_continue:;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "yarl/_quoting.pyx":237
 *         ret.append(ch)
 * 
 *     if pcts:             # <<<<<<<<<<<<<<
 *         try:
 *             unquoted = pcts.decode('utf8')
 */
  __pyx_t_18 = __Pyx_PyObject_IsTrue(__pyx_v_pcts); if (unlikely(__pyx_t_18 < 0)) __PYX_ERR(0, 237, __pyx_L1_error)
  if (__pyx_t_18) {

    /* "yarl/_quoting.pyx":238
 * 
 *     if pcts:
 *         try:             # <<<<<<<<<<<<<<
 *             unquoted = pcts.decode('utf8')
 *         except UnicodeDecodeError:
 */
    {
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __Pyx_ExceptionSave(&__pyx_t_16, &__pyx_t_15, &__pyx_t_14);
      __Pyx_XGOTREF(__pyx_t_16);
      __Pyx_XGOTREF(__pyx_t_15);
      __Pyx_XGOTREF(__pyx_t_14);
      /*try:*/ {

        /* "yarl/_quoting.pyx":239
 *     if pcts:
 *         try:
 *             unquoted = pcts.decode('utf8')             # <<<<<<<<<<<<<<
 *         except UnicodeDecodeError:
 *             ret.append(last_pct)  # %F8
 */
        __pyx_t_11 = __Pyx_decode_bytearray(__pyx_v_pcts, 0, PY_SSIZE_T_MAX, NULL, NULL, PyUnicode_DecodeUTF8); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 239, __pyx_L27_error)
        __Pyx_GOTREF(__pyx_t_11);
        if (!(likely(PyUnicode_CheckExact(__pyx_t_11))||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_t_11)->tp_name), 0))) __PYX_ERR(0, 239, __pyx_L27_error)
        __Pyx_XDECREF_SET(__pyx_v_unquoted, ((PyObject*)__pyx_t_11));
        __pyx_t_11 = 0;

        /* "yarl/_quoting.pyx":238
 * 
 *     if pcts:
 *         try:             # <<<<<<<<<<<<<<
 *             unquoted = pcts.decode('utf8')
 *         except UnicodeDecodeError:
 */
      }

      /* "yarl/_quoting.pyx":243
 *             ret.append(last_pct)  # %F8
 *         else:
 *             if qs and unquoted in '+=&;':             # <<<<<<<<<<<<<<
 *                 ret.append(_do_quote(unquoted, '', '', True, True))
 *             elif unquoted in unsafe:
 */
      /*else:*/ {
        __pyx_t_9 = (__pyx_v_qs != 0);
        if (__pyx_t_9) {
        } else {
          __pyx_t_18 = __pyx_t_9;
          goto __pyx_L36_bool_binop_done;
        }
        __pyx_t_9 = (__Pyx_PyUnicode_ContainsTF(__pyx_v_unquoted, __pyx_kp_u__7, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 243, __pyx_L29_except_error)
        __pyx_t_17 = (__pyx_t_9 != 0);
        __pyx_t_18 = __pyx_t_17;
        __pyx_L36_bool_binop_done:;
        if (__pyx_t_18) {

          /* "yarl/_quoting.pyx":244
 *         else:
 *             if qs and unquoted in '+=&;':
 *                 ret.append(_do_quote(unquoted, '', '', True, True))             # <<<<<<<<<<<<<<
 *             elif unquoted in unsafe:
 *                 ret.append(_do_quote(unquoted, '', '', False, True))
 */
          __pyx_t_11 = __pyx_f_4yarl_8_quoting__do_quote(__pyx_v_unquoted, __pyx_kp_u_, __pyx_kp_u_, 1, 1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 244, __pyx_L29_except_error)
          __Pyx_GOTREF(__pyx_t_11);
          __pyx_t_13 = __Pyx_PyList_Append(__pyx_v_ret, __pyx_t_11); if (unlikely(__pyx_t_13 == -1)) __PYX_ERR(0, 244, __pyx_L29_except_error)
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

          /* "yarl/_quoting.pyx":243
 *             ret.append(last_pct)  # %F8
 *         else:
 *             if qs and unquoted in '+=&;':             # <<<<<<<<<<<<<<
 *                 ret.append(_do_quote(unquoted, '', '', True, True))
 *             elif unquoted in unsafe:
 */
          goto __pyx_L35;
        }

        /* "yarl/_quoting.pyx":245
 *             if qs and unquoted in '+=&;':
 *                 ret.append(_do_quote(unquoted, '', '', True, True))
 *             elif unquoted in unsafe:             # <<<<<<<<<<<<<<
 *                 ret.append(_do_quote(unquoted, '', '', False, True))
 *             else:
 */
        if (unlikely(__pyx_v_unsafe == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
          __PYX_ERR(0, 245, __pyx_L29_except_error)
        }
        __pyx_t_18 = (__Pyx_PyUnicode_ContainsTF(__pyx_v_unquoted, __pyx_v_unsafe, Py_EQ)); if (unlikely(__pyx_t_18 < 0)) __PYX_ERR(0, 245, __pyx_L29_except_error)
        __pyx_t_17 = (__pyx_t_18 != 0);
        if (__pyx_t_17) {

          /* "yarl/_quoting.pyx":246
 *                 ret.append(_do_quote(unquoted, '', '', True, True))
 *             elif unquoted in unsafe:
 *                 ret.append(_do_quote(unquoted, '', '', False, True))             # <<<<<<<<<<<<<<
 *             else:
 *                 ret.append(unquoted)
 */
          __pyx_t_11 = __pyx_f_4yarl_8_quoting__do_quote(__pyx_v_unquoted, __pyx_kp_u_, __pyx_kp_u_, 0, 1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 246, __pyx_L29_except_error)
          __Pyx_GOTREF(__pyx_t_11);
          __pyx_t_13 = __Pyx_PyList_Append(__pyx_v_ret, __pyx_t_11); if (unlikely(__pyx_t_13 == -1)) __PYX_ERR(0, 246, __pyx_L29_except_error)
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

          /* "yarl/_quoting.pyx":245
 *             if qs and unquoted in '+=&;':
 *                 ret.append(_do_quote(unquoted, '', '', True, True))
 *             elif unquoted in unsafe:             # <<<<<<<<<<<<<<
 *                 ret.append(_do_quote(unquoted, '', '', False, True))
 *             else:
 */
          goto __pyx_L35;
        }

        /* "yarl/_quoting.pyx":248
 *                 ret.append(_do_quote(unquoted, '', '', False, True))
 *             else:
 *                 ret.append(unquoted)             # <<<<<<<<<<<<<<
 *     return ''.join(ret)
 */
        /*else*/ {
          __pyx_t_13 = __Pyx_PyList_Append(__pyx_v_ret, __pyx_v_unquoted); if (unlikely(__pyx_t_13 == -1)) __PYX_ERR(0, 248, __pyx_L29_except_error)
        }
        __pyx_L35:;
      }
      __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
      __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
      __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
      goto __pyx_L34_try_end;
      __pyx_L27_error:;
      __Pyx_PyThreadState_assign
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
      __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;

      /* "yarl/_quoting.pyx":240
 *         try:
 *             unquoted = pcts.decode('utf8')
 *         except UnicodeDecodeError:             # <<<<<<<<<<<<<<
 *             ret.append(last_pct)  # %F8
 *         else:
 */
      __pyx_t_6 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_UnicodeDecodeError);
      if (__pyx_t_6) {
        __Pyx_AddTraceback("yarl._quoting._do_unquote", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_11, &__pyx_t_12, &__pyx_t_1) < 0) __PYX_ERR(0, 240, __pyx_L29_except_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_GOTREF(__pyx_t_12);
        __Pyx_GOTREF(__pyx_t_1);

        /* "yarl/_quoting.pyx":241
 *             unquoted = pcts.decode('utf8')
 *         except UnicodeDecodeError:
 *             ret.append(last_pct)  # %F8             # <<<<<<<<<<<<<<
 *         else:
 *             if qs and unquoted in '+=&;':
 */
        __pyx_t_13 = __Pyx_PyList_Append(__pyx_v_ret, __pyx_v_last_pct); if (unlikely(__pyx_t_13 == -1)) __PYX_ERR(0, 241, __pyx_L29_except_error)
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L28_exception_handled;
      }
      goto __pyx_L29_except_error;
      __pyx_L29_except_error:;

      /* "yarl/_quoting.pyx":238
 * 
 *     if pcts:
 *         try:             # <<<<<<<<<<<<<<
 *             unquoted = pcts.decode('utf8')
 *         except UnicodeDecodeError:
 */
      __Pyx_PyThreadState_assign
      __Pyx_XGIVEREF(__pyx_t_16);
      __Pyx_XGIVEREF(__pyx_t_15);
      __Pyx_XGIVEREF(__pyx_t_14);
      __Pyx_ExceptionReset(__pyx_t_16, __pyx_t_15, __pyx_t_14);
      goto __pyx_L1_error;
      __pyx_L28_exception_handled:;
      __Pyx_PyThreadState_assign
      __Pyx_XGIVEREF(__pyx_t_16);
      __Pyx_XGIVEREF(__pyx_t_15);
      __Pyx_XGIVEREF(__pyx_t_14);
      __Pyx_ExceptionReset(__pyx_t_16, __pyx_t_15, __pyx_t_14);
      __pyx_L34_try_end:;
    }

    /* "yarl/_quoting.pyx":237
 *         ret.append(ch)
 * 
 *     if pcts:             # <<<<<<<<<<<<<<
 *         try:
 *             unquoted = pcts.decode('utf8')
 */
  }

  /* "yarl/_quoting.pyx":249
 *             else:
 *                 ret.append(unquoted)
 *     return ''.join(ret)             # <<<<<<<<<<<<<<
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyUnicode_Join(__pyx_kp_u_, __pyx_v_ret); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 249, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "yarl/_quoting.pyx":185
 * 
 * 
 * cdef str _do_unquote(str val, str unsafe='', bint qs=False):             # <<<<<<<<<<<<<<
 *     cdef str pct = ''
 *     cdef str last_pct = ''
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("yarl._quoting._do_unquote", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_pct);
  __Pyx_XDECREF(__pyx_v_last_pct);
  __Pyx_XDECREF(__pyx_v_pcts);
  __Pyx_XDECREF(__pyx_v_ret);
  __Pyx_XDECREF(__pyx_v_unquoted);
  __Pyx_XDECREF(__pyx_v_ch);
  __Pyx_XDECREF(__pyx_v_h);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
static struct PyModuleDef __pyx_moduledef = {
  #if PY_VERSION_HEX < 0x03020000
    { PyObject_HEAD_INIT(NULL) NULL, 0, NULL },
  #else
    PyModuleDef_HEAD_INIT,
  #endif
    "_quoting",
    0, /* m_doc */
    -1, /* m_size */
    __pyx_methods /* m_methods */,
    NULL, /* m_reload */
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_u_, __pyx_k_, sizeof(__pyx_k_), 0, 1, 0, 0},
  {&__pyx_kp_u_Argument_should_be_str, __pyx_k_Argument_should_be_str, sizeof(__pyx_k_Argument_should_be_str), 0, 1, 0, 0},
  {&__pyx_n_s_TypeError, __pyx_k_TypeError, sizeof(__pyx_k_TypeError), 0, 0, 1, 1},
  {&__pyx_kp_u_Unallowed_PCT, __pyx_k_Unallowed_PCT, sizeof(__pyx_k_Unallowed_PCT), 0, 1, 0, 0},
  {&__pyx_kp_u_Unallowed_PCT_2, __pyx_k_Unallowed_PCT_2, sizeof(__pyx_k_Unallowed_PCT_2), 0, 1, 0, 0},
  {&__pyx_kp_u_Unallowed_PCT_3, __pyx_k_Unallowed_PCT_3, sizeof(__pyx_k_Unallowed_PCT_3), 0, 1, 0, 0},
  {&__pyx_n_s_UnicodeDecodeError, __pyx_k_UnicodeDecodeError, sizeof(__pyx_k_UnicodeDecodeError), 0, 0, 1, 1},
  {&__pyx_n_s_ValueError, __pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 0, 1, 1},
  {&__pyx_kp_u__10, __pyx_k__10, sizeof(__pyx_k__10), 0, 1, 0, 0},
  {&__pyx_kp_u__12, __pyx_k__12, sizeof(__pyx_k__12), 0, 1, 0, 0},
  {&__pyx_kp_u__13, __pyx_k__13, sizeof(__pyx_k__13), 0, 1, 0, 0},
  {&__pyx_kp_u__14, __pyx_k__14, sizeof(__pyx_k__14), 0, 1, 0, 0},
  {&__pyx_kp_u__15, __pyx_k__15, sizeof(__pyx_k__15), 0, 1, 0, 0},
  {&__pyx_kp_u__3, __pyx_k__3, sizeof(__pyx_k__3), 0, 1, 0, 0},
  {&__pyx_kp_u__7, __pyx_k__7, sizeof(__pyx_k__7), 0, 1, 0, 0},
  {&__pyx_kp_u__8, __pyx_k__8, sizeof(__pyx_k__8), 0, 1, 0, 0},
  {&__pyx_kp_u__9, __pyx_k__9, sizeof(__pyx_k__9), 0, 1, 0, 0},
  {&__pyx_n_s_append, __pyx_k_append, sizeof(__pyx_k_append), 0, 0, 1, 1},
  {&__pyx_n_s_ascii_letters, __pyx_k_ascii_letters, sizeof(__pyx_k_ascii_letters), 0, 0, 1, 1},
  {&__pyx_n_s_base, __pyx_k_base, sizeof(__pyx_k_base), 0, 0, 1, 1},
  {&__pyx_n_s_digits, __pyx_k_digits, sizeof(__pyx_k_digits), 0, 0, 1, 1},
  {&__pyx_n_s_encode, __pyx_k_encode, sizeof(__pyx_k_encode), 0, 0, 1, 1},
  {&__pyx_n_s_errors, __pyx_k_errors, sizeof(__pyx_k_errors), 0, 0, 1, 1},
  {&__pyx_n_s_format, __pyx_k_format, sizeof(__pyx_k_format), 0, 0, 1, 1},
  {&__pyx_n_s_hex, __pyx_k_hex, sizeof(__pyx_k_hex), 0, 0, 1, 1},
  {&__pyx_kp_s_home_travis_build_aio_libs_yarl, __pyx_k_home_travis_build_aio_libs_yarl, sizeof(__pyx_k_home_travis_build_aio_libs_yarl), 0, 0, 1, 0},
  {&__pyx_n_u_ignore, __pyx_k_ignore, sizeof(__pyx_k_ignore), 0, 1, 0, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_protected, __pyx_k_protected, sizeof(__pyx_k_protected), 0, 0, 1, 1},
  {&__pyx_n_s_qs, __pyx_k_qs, sizeof(__pyx_k_qs), 0, 0, 1, 1},
  {&__pyx_n_s_quote, __pyx_k_quote, sizeof(__pyx_k_quote), 0, 0, 1, 1},
  {&__pyx_n_s_safe, __pyx_k_safe, sizeof(__pyx_k_safe), 0, 0, 1, 1},
  {&__pyx_n_s_strict, __pyx_k_strict, sizeof(__pyx_k_strict), 0, 0, 1, 1},
  {&__pyx_n_u_strict, __pyx_k_strict, sizeof(__pyx_k_strict), 0, 1, 0, 1},
  {&__pyx_n_s_string, __pyx_k_string, sizeof(__pyx_k_string), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_unquote, __pyx_k_unquote, sizeof(__pyx_k_unquote), 0, 0, 1, 1},
  {&__pyx_n_s_unsafe, __pyx_k_unsafe, sizeof(__pyx_k_unsafe), 0, 0, 1, 1},
  {&__pyx_n_s_upper, __pyx_k_upper, sizeof(__pyx_k_upper), 0, 0, 1, 1},
  {&__pyx_kp_u_utf_8, __pyx_k_utf_8, sizeof(__pyx_k_utf_8), 0, 1, 0, 0},
  {&__pyx_n_s_val, __pyx_k_val, sizeof(__pyx_k_val), 0, 0, 1, 1},
  {&__pyx_n_s_yarl__quoting, __pyx_k_yarl__quoting, sizeof(__pyx_k_yarl__quoting), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_TypeError = __Pyx_GetBuiltinName(__pyx_n_s_TypeError); if (!__pyx_builtin_TypeError) __PYX_ERR(0, 42, __pyx_L1_error)
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_n_s_ValueError); if (!__pyx_builtin_ValueError) __PYX_ERR(0, 78, __pyx_L1_error)
  __pyx_builtin_UnicodeDecodeError = __Pyx_GetBuiltinName(__pyx_n_s_UnicodeDecodeError); if (!__pyx_builtin_UnicodeDecodeError) __PYX_ERR(0, 202, __pyx_L1_error)
  __pyx_builtin_hex = __Pyx_GetBuiltinName(__pyx_n_s_hex); if (!__pyx_builtin_hex) __PYX_ERR(0, 230, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "yarl/_quoting.pyx":42
 *         return None
 *     if not isinstance(val, str):
 *         raise TypeError("Argument should be str")             # <<<<<<<<<<<<<<
 *     if not val:
 *         return ''
 */
  __pyx_tuple__2 = PyTuple_Pack(1, __pyx_kp_u_Argument_should_be_str); if (unlikely(!__pyx_tuple__2)) __PYX_ERR(0, 42, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);

  /* "yarl/_quoting.pyx":135
 *             if idx == len(val):
 *                 if strict:
 *                     raise ValueError("Unallowed PCT %")             # <<<<<<<<<<<<<<
 *                 else:
 *                     PyUnicode_WriteChar(ret, ret_idx, '%')
 */
  __pyx_tuple__4 = PyTuple_Pack(1, __pyx_kp_u_Unallowed_PCT_3); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(0, 135, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);

  /* "yarl/_quoting.pyx":160
 *             continue
 * 
 *         ch_bytes = ch.encode("utf-8", errors= 'strict' if strict else 'ignore')             # <<<<<<<<<<<<<<
 * 
 *         for b in ch_bytes:
 */
  __pyx_tuple__5 = PyTuple_Pack(1, __pyx_kp_u_utf_8); if (unlikely(!__pyx_tuple__5)) __PYX_ERR(0, 160, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__5);
  __Pyx_GIVEREF(__pyx_tuple__5);

  /* "yarl/_quoting.pyx":179
 *         return None
 *     if not isinstance(val, str):
 *         raise TypeError("Argument should be str")             # <<<<<<<<<<<<<<
 *     if not val:
 *         return ''
 */
  __pyx_tuple__6 = PyTuple_Pack(1, __pyx_kp_u_Argument_should_be_str); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(0, 179, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);

  /* "yarl/_quoting.pyx":230
 *         if ch in unsafe:
 *             ret.append('%')
 *             h = hex(ord(ch)).upper()[2:]             # <<<<<<<<<<<<<<
 *             for ch in h:
 *                 ret.append(ch)
 */
  __pyx_slice__11 = PySlice_New(__pyx_int_2, Py_None, Py_None); if (unlikely(!__pyx_slice__11)) __PYX_ERR(0, 230, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__11);
  __Pyx_GIVEREF(__pyx_slice__11);

  /* "yarl/_quoting.pyx":38
 * 
 * 
 * def _quote(val, *, str safe='', str protected='', bint qs=False, bint strict=True):             # <<<<<<<<<<<<<<
 *     if val is None:
 *         return None
 */
  __pyx_tuple__16 = PyTuple_Pack(5, __pyx_n_s_val, __pyx_n_s_safe, __pyx_n_s_protected, __pyx_n_s_qs, __pyx_n_s_strict); if (unlikely(!__pyx_tuple__16)) __PYX_ERR(0, 38, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__16);
  __Pyx_GIVEREF(__pyx_tuple__16);
  __pyx_codeobj__17 = (PyObject*)__Pyx_PyCode_New(1, 4, 5, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__16, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_travis_build_aio_libs_yarl, __pyx_n_s_quote, 38, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__17)) __PYX_ERR(0, 38, __pyx_L1_error)

  /* "yarl/_quoting.pyx":175
 * 
 * 
 * def _unquote(val, *, unsafe='', qs=False):             # <<<<<<<<<<<<<<
 *     if val is None:
 *         return None
 */
  __pyx_tuple__18 = PyTuple_Pack(3, __pyx_n_s_val, __pyx_n_s_unsafe, __pyx_n_s_qs); if (unlikely(!__pyx_tuple__18)) __PYX_ERR(0, 175, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__18);
  __Pyx_GIVEREF(__pyx_tuple__18);
  __pyx_codeobj__19 = (PyObject*)__Pyx_PyCode_New(1, 2, 3, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__18, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_home_travis_build_aio_libs_yarl, __pyx_n_s_unquote, 175, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__19)) __PYX_ERR(0, 175, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_int_2 = PyInt_FromLong(2); if (unlikely(!__pyx_int_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_16 = PyInt_FromLong(16); if (unlikely(!__pyx_int_16)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

#if PY_MAJOR_VERSION < 3
PyMODINIT_FUNC init_quoting(void); /*proto*/
PyMODINIT_FUNC init_quoting(void)
#else
PyMODINIT_FUNC PyInit__quoting(void); /*proto*/
PyMODINIT_FUNC PyInit__quoting(void)
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannyDeclarations
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
      PyErr_Clear();
      __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
      if (!__Pyx_RefNanny)
          Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  __Pyx_RefNannySetupContext("PyMODINIT_FUNC PyInit__quoting(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("_quoting", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_yarl___quoting) {
    if (PyObject_SetAttrString(__pyx_m, "__name__", __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "yarl._quoting")) {
      if (unlikely(PyDict_SetItemString(modules, "yarl._quoting", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global init code ---*/
  __pyx_v_4yarl_8_quoting_GEN_DELIMS = ((PyObject*)Py_None); Py_INCREF(Py_None);
  __pyx_v_4yarl_8_quoting_SUB_DELIMS_WITHOUT_QS = ((PyObject*)Py_None); Py_INCREF(Py_None);
  __pyx_v_4yarl_8_quoting_SUB_DELIMS = ((PyObject*)Py_None); Py_INCREF(Py_None);
  __pyx_v_4yarl_8_quoting_RESERVED = ((PyObject*)Py_None); Py_INCREF(Py_None);
  __pyx_v_4yarl_8_quoting_UNRESERVED = ((PyObject*)Py_None); Py_INCREF(Py_None);
  __pyx_v_4yarl_8_quoting_ALLOWED = ((PyObject*)Py_None); Py_INCREF(Py_None);
  /*--- Variable export code ---*/
  /*--- Function export code ---*/
  /*--- Type init code ---*/
  /*--- Type import code ---*/
  /*--- Variable import code ---*/
  /*--- Function import code ---*/
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "yarl/_quoting.pyx":10
 * from libc.stdint cimport uint8_t, uint64_t
 * 
 * from string import ascii_letters, digits             # <<<<<<<<<<<<<<
 * 
 * cdef str GEN_DELIMS = ":/?#[]@"
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_ascii_letters);
  __Pyx_GIVEREF(__pyx_n_s_ascii_letters);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_ascii_letters);
  __Pyx_INCREF(__pyx_n_s_digits);
  __Pyx_GIVEREF(__pyx_n_s_digits);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_s_digits);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_string, __pyx_t_1, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_ascii_letters); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ascii_letters, __pyx_t_1) < 0) __PYX_ERR(0, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_digits); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_digits, __pyx_t_1) < 0) __PYX_ERR(0, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "yarl/_quoting.pyx":12
 * from string import ascii_letters, digits
 * 
 * cdef str GEN_DELIMS = ":/?#[]@"             # <<<<<<<<<<<<<<
 * cdef str SUB_DELIMS_WITHOUT_QS = "!$'()*,"
 * cdef str SUB_DELIMS = SUB_DELIMS_WITHOUT_QS + '+?=;'
 */
  __Pyx_INCREF(__pyx_kp_u__12);
  __Pyx_XGOTREF(__pyx_v_4yarl_8_quoting_GEN_DELIMS);
  __Pyx_DECREF_SET(__pyx_v_4yarl_8_quoting_GEN_DELIMS, __pyx_kp_u__12);
  __Pyx_GIVEREF(__pyx_kp_u__12);

  /* "yarl/_quoting.pyx":13
 * 
 * cdef str GEN_DELIMS = ":/?#[]@"
 * cdef str SUB_DELIMS_WITHOUT_QS = "!$'()*,"             # <<<<<<<<<<<<<<
 * cdef str SUB_DELIMS = SUB_DELIMS_WITHOUT_QS + '+?=;'
 * cdef str RESERVED = GEN_DELIMS + SUB_DELIMS
 */
  __Pyx_INCREF(__pyx_kp_u__13);
  __Pyx_XGOTREF(__pyx_v_4yarl_8_quoting_SUB_DELIMS_WITHOUT_QS);
  __Pyx_DECREF_SET(__pyx_v_4yarl_8_quoting_SUB_DELIMS_WITHOUT_QS, __pyx_kp_u__13);
  __Pyx_GIVEREF(__pyx_kp_u__13);

  /* "yarl/_quoting.pyx":14
 * cdef str GEN_DELIMS = ":/?#[]@"
 * cdef str SUB_DELIMS_WITHOUT_QS = "!$'()*,"
 * cdef str SUB_DELIMS = SUB_DELIMS_WITHOUT_QS + '+?=;'             # <<<<<<<<<<<<<<
 * cdef str RESERVED = GEN_DELIMS + SUB_DELIMS
 * cdef str UNRESERVED = ascii_letters + digits + '-._~'
 */
  __pyx_t_2 = __Pyx_PyUnicode_ConcatSafe(__pyx_v_4yarl_8_quoting_SUB_DELIMS_WITHOUT_QS, __pyx_kp_u__14); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 14, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_XGOTREF(__pyx_v_4yarl_8_quoting_SUB_DELIMS);
  __Pyx_DECREF_SET(__pyx_v_4yarl_8_quoting_SUB_DELIMS, ((PyObject*)__pyx_t_2));
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "yarl/_quoting.pyx":15
 * cdef str SUB_DELIMS_WITHOUT_QS = "!$'()*,"
 * cdef str SUB_DELIMS = SUB_DELIMS_WITHOUT_QS + '+?=;'
 * cdef str RESERVED = GEN_DELIMS + SUB_DELIMS             # <<<<<<<<<<<<<<
 * cdef str UNRESERVED = ascii_letters + digits + '-._~'
 * cdef str ALLOWED = UNRESERVED + SUB_DELIMS_WITHOUT_QS
 */
  __pyx_t_2 = __Pyx_PyUnicode_ConcatSafe(__pyx_v_4yarl_8_quoting_GEN_DELIMS, __pyx_v_4yarl_8_quoting_SUB_DELIMS); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_XGOTREF(__pyx_v_4yarl_8_quoting_RESERVED);
  __Pyx_DECREF_SET(__pyx_v_4yarl_8_quoting_RESERVED, ((PyObject*)__pyx_t_2));
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "yarl/_quoting.pyx":16
 * cdef str SUB_DELIMS = SUB_DELIMS_WITHOUT_QS + '+?=;'
 * cdef str RESERVED = GEN_DELIMS + SUB_DELIMS
 * cdef str UNRESERVED = ascii_letters + digits + '-._~'             # <<<<<<<<<<<<<<
 * cdef str ALLOWED = UNRESERVED + SUB_DELIMS_WITHOUT_QS
 * 
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_ascii_letters); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_digits); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyNumber_Add(__pyx_t_2, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyNumber_Add(__pyx_t_3, __pyx_kp_u__15); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "unicode", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 16, __pyx_L1_error)
  __Pyx_XGOTREF(__pyx_v_4yarl_8_quoting_UNRESERVED);
  __Pyx_DECREF_SET(__pyx_v_4yarl_8_quoting_UNRESERVED, ((PyObject*)__pyx_t_1));
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;

  /* "yarl/_quoting.pyx":17
 * cdef str RESERVED = GEN_DELIMS + SUB_DELIMS
 * cdef str UNRESERVED = ascii_letters + digits + '-._~'
 * cdef str ALLOWED = UNRESERVED + SUB_DELIMS_WITHOUT_QS             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __Pyx_PyUnicode_ConcatSafe(__pyx_v_4yarl_8_quoting_UNRESERVED, __pyx_v_4yarl_8_quoting_SUB_DELIMS_WITHOUT_QS); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_XGOTREF(__pyx_v_4yarl_8_quoting_ALLOWED);
  __Pyx_DECREF_SET(__pyx_v_4yarl_8_quoting_ALLOWED, ((PyObject*)__pyx_t_1));
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;

  /* "yarl/_quoting.pyx":38
 * 
 * 
 * def _quote(val, *, str safe='', str protected='', bint qs=False, bint strict=True):             # <<<<<<<<<<<<<<
 *     if val is None:
 *         return None
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_4yarl_8_quoting_1_quote, NULL, __pyx_n_s_yarl__quoting); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 38, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_quote, __pyx_t_1) < 0) __PYX_ERR(0, 38, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "yarl/_quoting.pyx":175
 * 
 * 
 * def _unquote(val, *, unsafe='', qs=False):             # <<<<<<<<<<<<<<
 *     if val is None:
 *         return None
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_4yarl_8_quoting_3_unquote, NULL, __pyx_n_s_yarl__quoting); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 175, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_unquote, __pyx_t_1) < 0) __PYX_ERR(0, 175, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "yarl/_quoting.pyx":1
 * # cython: language_level=3             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "Python.h":
 */
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_1) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init yarl._quoting", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_DECREF(__pyx_m); __pyx_m = 0;
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init yarl._quoting");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if PY_MAJOR_VERSION < 3
  return;
  #else
  return __pyx_m;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule((char *)modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, (char *)"RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_CheckExact(key)) || likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (PyUnicode_GET_SIZE(**name) != PyUnicode_GET_SIZE(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (PyUnicode_GET_SIZE(**argname) != PyUnicode_GET_SIZE(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* ArgTypeTest */
static void __Pyx_RaiseArgumentTypeInvalid(const char* name, PyObject *obj, PyTypeObject *type) {
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
}
static CYTHON_INLINE int __Pyx_ArgTypeTest(PyObject *obj, PyTypeObject *type, int none_allowed,
    const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (none_allowed && obj == Py_None) return 1;
    else if (exact) {
        if (likely(Py_TYPE(obj) == type)) return 1;
        #if PY_MAJOR_VERSION == 2
        else if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(PyObject_TypeCheck(obj, type))) return 1;
    }
    __Pyx_RaiseArgumentTypeInvalid(name, obj, type);
    return 0;
}

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
#if PY_VERSION_HEX >= 0x03030000
    if (cause) {
#else
    if (cause && cause != Py_None) {
#endif
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = PyThreadState_GET();
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* GetItemIntUnicode */
  static CYTHON_INLINE Py_UCS4 __Pyx_GetItemInt_Unicode_Fast(PyObject* ustring, Py_ssize_t i,
                                                           int wraparound, int boundscheck) {
    Py_ssize_t length;
    if (unlikely(__Pyx_PyUnicode_READY(ustring) < 0)) return (Py_UCS4)-1;
    if (wraparound | boundscheck) {
        length = __Pyx_PyUnicode_GET_LENGTH(ustring);
        if (wraparound & unlikely(i < 0)) i += length;
        if ((!boundscheck) || likely((0 <= i) & (i < length))) {
            return __Pyx_PyUnicode_READ_CHAR(ustring, i);
        } else {
            PyErr_SetString(PyExc_IndexError, "string index out of range");
            return (Py_UCS4)-1;
        }
    } else {
        return __Pyx_PyUnicode_READ_CHAR(ustring, i);
    }
}

/* PyFunctionFastCall */
  #if CYTHON_FAST_PYCALL
#include "frameobject.h"
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = PyThreadState_GET();
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = f->f_localsplus;
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif  // CPython < 3.6
#endif  // CYTHON_FAST_PYCALL

/* PyCFunctionFastCall */
  #if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    return (*((__Pyx_PyCFunctionFast)meth)) (self, args, nargs, NULL);
}
#endif  // CYTHON_FAST_PYCCALL

/* PyObjectCallMethO */
  #if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallOneArg */
  #if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || PyObject_TypeCheck(func, __pyx_CyFunctionType))) {
#else
    if (likely(PyCFunction_Check(func))) {
#endif
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (PyCFunction_GET_FLAGS(func) & METH_FASTCALL) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* unicode_iter */
    static CYTHON_INLINE int __Pyx_init_unicode_iteration(
    PyObject* ustring, Py_ssize_t *length, void** data, int *kind) {
#if CYTHON_PEP393_ENABLED
    if (unlikely(__Pyx_PyUnicode_READY(ustring) < 0)) return -1;
    *kind   = PyUnicode_KIND(ustring);
    *length = PyUnicode_GET_LENGTH(ustring);
    *data   = PyUnicode_DATA(ustring);
#else
    *kind   = 0;
    *length = PyUnicode_GET_SIZE(ustring);
    *data   = (void*)PyUnicode_AS_UNICODE(ustring);
#endif
    return 0;
}

/* PyObjectCallMethod1 */
    static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg) {
    PyObject *method, *result = NULL;
    method = __Pyx_PyObject_GetAttrStr(obj, method_name);
    if (unlikely(!method)) goto done;
#if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(method))) {
        PyObject *self = PyMethod_GET_SELF(method);
        if (likely(self)) {
            PyObject *args;
            PyObject *function = PyMethod_GET_FUNCTION(method);
            #if CYTHON_FAST_PYCALL
            if (PyFunction_Check(function)) {
                PyObject *args[2] = {self, arg};
                result = __Pyx_PyFunction_FastCall(function, args, 2);
                goto done;
            }
            #endif
            #if CYTHON_FAST_PYCCALL
            if (__Pyx_PyFastCFunction_Check(function)) {
                PyObject *args[2] = {self, arg};
                result = __Pyx_PyCFunction_FastCall(function, args, 2);
                goto done;
            }
            #endif
            args = PyTuple_New(2);
            if (unlikely(!args)) goto done;
            Py_INCREF(self);
            PyTuple_SET_ITEM(args, 0, self);
            Py_INCREF(arg);
            PyTuple_SET_ITEM(args, 1, arg);
            Py_INCREF(function);
            Py_DECREF(method); method = NULL;
            result = __Pyx_PyObject_Call(function, args, NULL);
            Py_DECREF(args);
            Py_DECREF(function);
            return result;
        }
    }
#endif
    result = __Pyx_PyObject_CallOneArg(method, arg);
done:
    Py_XDECREF(method);
    return result;
}

/* ByteArrayAppend */
    static CYTHON_INLINE int __Pyx_PyByteArray_Append(PyObject* bytearray, int value) {
    PyObject *pyval, *retval;
#if CYTHON_COMPILING_IN_CPYTHON
    if (likely((value >= 0) & (value <= 255))) {
        Py_ssize_t n = Py_SIZE(bytearray);
        if (likely(n != PY_SSIZE_T_MAX)) {
            if (unlikely(PyByteArray_Resize(bytearray, n + 1) < 0))
                return -1;
            PyByteArray_AS_STRING(bytearray)[n] = value;
            return 0;
        }
    } else {
        PyErr_SetString(PyExc_ValueError, "byte must be in range(0, 256)");
        return -1;
    }
#endif
    pyval = PyInt_FromLong(value);
    if (unlikely(!pyval))
        return -1;
    retval = __Pyx_PyObject_CallMethod1(bytearray, __pyx_n_s_append, pyval);
    Py_DECREF(pyval);
    if (unlikely(!retval))
        return -1;
    Py_DECREF(retval);
    return 0;
}

/* ByteArrayAppendObject */
    static CYTHON_INLINE int __Pyx_PyByteArray_AppendObject(PyObject* bytearray, PyObject* value) {
    Py_ssize_t ival;
#if PY_MAJOR_VERSION < 3
    if (unlikely(PyString_Check(value))) {
        if (unlikely(PyString_GET_SIZE(value) != 1)) {
            PyErr_SetString(PyExc_ValueError, "string must be of size 1");
            return -1;
        }
        ival = (unsigned char) (PyString_AS_STRING(value)[0]);
    } else
#endif
#if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(value)) && likely(Py_SIZE(value) == 1 || Py_SIZE(value) == 0)) {
        if (Py_SIZE(value) == 0) {
            ival = 0;
        } else {
            ival = ((PyLongObject*)value)->ob_digit[0];
            if (unlikely(ival > 255)) goto bad_range;
        }
    } else
#endif
    {
        ival = __Pyx_PyIndex_AsSsize_t(value);
        if (unlikely((ival < 0) | (ival > 255))) {
            if (ival == -1 && PyErr_Occurred())
                return -1;
            goto bad_range;
        }
    }
    return __Pyx_PyByteArray_Append(bytearray, ival);
bad_range:
    PyErr_SetString(PyExc_ValueError, "byte must be in range(0, 256)");
    return -1;
}

/* PyUnicode_Substring */
    static CYTHON_INLINE PyObject* __Pyx_PyUnicode_Substring(
            PyObject* text, Py_ssize_t start, Py_ssize_t stop) {
    Py_ssize_t length;
    if (unlikely(__Pyx_PyUnicode_READY(text) == -1)) return NULL;
    length = __Pyx_PyUnicode_GET_LENGTH(text);
    if (start < 0) {
        start += length;
        if (start < 0)
            start = 0;
    }
    if (stop < 0)
        stop += length;
    else if (stop > length)
        stop = length;
    length = stop - start;
    if (length <= 0)
        return PyUnicode_FromUnicode(NULL, 0);
#if CYTHON_PEP393_ENABLED
    return PyUnicode_FromKindAndData(PyUnicode_KIND(text),
        PyUnicode_1BYTE_DATA(text) + start*PyUnicode_KIND(text), stop-start);
#else
    return PyUnicode_FromUnicode(PyUnicode_AS_UNICODE(text)+start, stop-start);
#endif
}

/* decode_c_bytes */
    static CYTHON_INLINE PyObject* __Pyx_decode_c_bytes(
         const char* cstring, Py_ssize_t length, Py_ssize_t start, Py_ssize_t stop,
         const char* encoding, const char* errors,
         PyObject* (*decode_func)(const char *s, Py_ssize_t size, const char *errors)) {
    if (unlikely((start < 0) | (stop < 0))) {
        if (start < 0) {
            start += length;
            if (start < 0)
                start = 0;
        }
        if (stop < 0)
            stop += length;
    }
    if (stop > length)
        stop = length;
    length = stop - start;
    if (unlikely(length <= 0))
        return PyUnicode_FromUnicode(NULL, 0);
    cstring += start;
    if (decode_func) {
        return decode_func(cstring, length, errors);
    } else {
        return PyUnicode_Decode(cstring, length, encoding, errors);
    }
}

/* SaveResetException */
    #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
#endif

/* SliceObject */
    static CYTHON_INLINE int __Pyx_PyObject_SetSlice(PyObject* obj, PyObject* value,
        Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** _py_start, PyObject** _py_stop, PyObject** _py_slice,
        int has_cstart, int has_cstop, CYTHON_UNUSED int wraparound) {
#if CYTHON_USE_TYPE_SLOTS
    PyMappingMethods* mp;
#if PY_MAJOR_VERSION < 3
    PySequenceMethods* ms = Py_TYPE(obj)->tp_as_sequence;
    if (likely(ms && ms->sq_ass_slice)) {
        if (!has_cstart) {
            if (_py_start && (*_py_start != Py_None)) {
                cstart = __Pyx_PyIndex_AsSsize_t(*_py_start);
                if ((cstart == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstart = 0;
        }
        if (!has_cstop) {
            if (_py_stop && (*_py_stop != Py_None)) {
                cstop = __Pyx_PyIndex_AsSsize_t(*_py_stop);
                if ((cstop == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstop = PY_SSIZE_T_MAX;
        }
        if (wraparound && unlikely((cstart < 0) | (cstop < 0)) && likely(ms->sq_length)) {
            Py_ssize_t l = ms->sq_length(obj);
            if (likely(l >= 0)) {
                if (cstop < 0) {
                    cstop += l;
                    if (cstop < 0) cstop = 0;
                }
                if (cstart < 0) {
                    cstart += l;
                    if (cstart < 0) cstart = 0;
                }
            } else {
                if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                    goto bad;
                PyErr_Clear();
            }
        }
        return ms->sq_ass_slice(obj, cstart, cstop, value);
    }
#endif
    mp = Py_TYPE(obj)->tp_as_mapping;
    if (likely(mp && mp->mp_ass_subscript))
#endif
    {
        int result;
        PyObject *py_slice, *py_start, *py_stop;
        if (_py_slice) {
            py_slice = *_py_slice;
        } else {
            PyObject* owned_start = NULL;
            PyObject* owned_stop = NULL;
            if (_py_start) {
                py_start = *_py_start;
            } else {
                if (has_cstart) {
                    owned_start = py_start = PyInt_FromSsize_t(cstart);
                    if (unlikely(!py_start)) goto bad;
                } else
                    py_start = Py_None;
            }
            if (_py_stop) {
                py_stop = *_py_stop;
            } else {
                if (has_cstop) {
                    owned_stop = py_stop = PyInt_FromSsize_t(cstop);
                    if (unlikely(!py_stop)) {
                        Py_XDECREF(owned_start);
                        goto bad;
                    }
                } else
                    py_stop = Py_None;
            }
            py_slice = PySlice_New(py_start, py_stop, Py_None);
            Py_XDECREF(owned_start);
            Py_XDECREF(owned_stop);
            if (unlikely(!py_slice)) goto bad;
        }
#if CYTHON_USE_TYPE_SLOTS
        result = mp->mp_ass_subscript(obj, py_slice, value);
#else
        result = value ? PyObject_SetItem(obj, py_slice, value) : PyObject_DelItem(obj, py_slice);
#endif
        if (!_py_slice) {
            Py_DECREF(py_slice);
        }
        return result;
    }
    PyErr_Format(PyExc_TypeError,
        "'%.200s' object does not support slice %.10s",
        Py_TYPE(obj)->tp_name, value ? "assignment" : "deletion");
bad:
    return -1;
}

/* PyErrExceptionMatches */
    #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    PyObject *exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
    return PyErr_GivenExceptionMatches(exc_type, err);
}
#endif

/* BytesEquals */
    static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals */
    static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
#if PY_MAJOR_VERSION < 3
    PyObject* owned_ref = NULL;
#endif
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
#if PY_MAJOR_VERSION < 3
    if ((s1_is_unicode & (!s2_is_unicode)) && PyString_CheckExact(s2)) {
        owned_ref = PyUnicode_FromObject(s2);
        if (unlikely(!owned_ref))
            return -1;
        s2 = owned_ref;
        s2_is_unicode = 1;
    } else if ((s2_is_unicode & (!s1_is_unicode)) && PyString_CheckExact(s1)) {
        owned_ref = PyUnicode_FromObject(s1);
        if (unlikely(!owned_ref))
            return -1;
        s1 = owned_ref;
        s1_is_unicode = 1;
    } else if (((!s2_is_unicode) & (!s1_is_unicode))) {
        return __Pyx_PyBytes_Equals(s1, s2, equals);
    }
#endif
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length;
        int kind;
        void *data1, *data2;
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        if (length != __Pyx_PyUnicode_GET_LENGTH(s2)) {
            goto return_ne;
        }
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            #if PY_MAJOR_VERSION < 3
            Py_XDECREF(owned_ref);
            #endif
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_EQ);
return_ne:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_NE);
#endif
}

/* UnicodeAsUCS4 */
    static CYTHON_INLINE Py_UCS4 __Pyx_PyUnicode_AsPy_UCS4(PyObject* x) {
   Py_ssize_t length;
   #if CYTHON_PEP393_ENABLED
   length = PyUnicode_GET_LENGTH(x);
   if (likely(length == 1)) {
       return PyUnicode_READ_CHAR(x, 0);
   }
   #else
   length = PyUnicode_GET_SIZE(x);
   if (likely(length == 1)) {
       return PyUnicode_AS_UNICODE(x)[0];
   }
   #if Py_UNICODE_SIZE == 2
   else if (PyUnicode_GET_SIZE(x) == 2) {
       Py_UCS4 high_val = PyUnicode_AS_UNICODE(x)[0];
       if (high_val >= 0xD800 && high_val <= 0xDBFF) {
           Py_UCS4 low_val = PyUnicode_AS_UNICODE(x)[1];
           if (low_val >= 0xDC00 && low_val <= 0xDFFF) {
               return 0x10000 + (((high_val & ((1<<10)-1)) << 10) | (low_val & ((1<<10)-1)));
           }
       }
   }
   #endif
   #endif
   PyErr_Format(PyExc_ValueError,
                "only single character unicode strings can be converted to Py_UCS4, "
                "got length %" CYTHON_FORMAT_SSIZE_T "d", length);
   return (Py_UCS4)-1;
}

/* object_ord */
    static long __Pyx__PyObject_Ord(PyObject* c) {
    Py_ssize_t size;
    if (PyBytes_Check(c)) {
        size = PyBytes_GET_SIZE(c);
        if (likely(size == 1)) {
            return (unsigned char) PyBytes_AS_STRING(c)[0];
        }
#if PY_MAJOR_VERSION < 3
    } else if (PyUnicode_Check(c)) {
        return (long)__Pyx_PyUnicode_AsPy_UCS4(c);
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    } else if (PyByteArray_Check(c)) {
        size = PyByteArray_GET_SIZE(c);
        if (likely(size == 1)) {
            return (unsigned char) PyByteArray_AS_STRING(c)[0];
        }
#endif
    } else {
        PyErr_Format(PyExc_TypeError,
            "ord() expected string of length 1, but %.200s found", c->ob_type->tp_name);
        return (long)(Py_UCS4)-1;
    }
    PyErr_Format(PyExc_TypeError,
        "ord() expected a character, but string of length %zd found", size);
    return (long)(Py_UCS4)-1;
}

/* PyObjectCallNoArg */
    #if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || PyObject_TypeCheck(func, __pyx_CyFunctionType))) {
#else
    if (likely(PyCFunction_Check(func))) {
#endif
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* SliceObject */
      static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(PyObject* obj,
        Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** _py_start, PyObject** _py_stop, PyObject** _py_slice,
        int has_cstart, int has_cstop, CYTHON_UNUSED int wraparound) {
#if CYTHON_USE_TYPE_SLOTS
    PyMappingMethods* mp;
#if PY_MAJOR_VERSION < 3
    PySequenceMethods* ms = Py_TYPE(obj)->tp_as_sequence;
    if (likely(ms && ms->sq_slice)) {
        if (!has_cstart) {
            if (_py_start && (*_py_start != Py_None)) {
                cstart = __Pyx_PyIndex_AsSsize_t(*_py_start);
                if ((cstart == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstart = 0;
        }
        if (!has_cstop) {
            if (_py_stop && (*_py_stop != Py_None)) {
                cstop = __Pyx_PyIndex_AsSsize_t(*_py_stop);
                if ((cstop == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstop = PY_SSIZE_T_MAX;
        }
        if (wraparound && unlikely((cstart < 0) | (cstop < 0)) && likely(ms->sq_length)) {
            Py_ssize_t l = ms->sq_length(obj);
            if (likely(l >= 0)) {
                if (cstop < 0) {
                    cstop += l;
                    if (cstop < 0) cstop = 0;
                }
                if (cstart < 0) {
                    cstart += l;
                    if (cstart < 0) cstart = 0;
                }
            } else {
                if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                    goto bad;
                PyErr_Clear();
            }
        }
        return ms->sq_slice(obj, cstart, cstop);
    }
#endif
    mp = Py_TYPE(obj)->tp_as_mapping;
    if (likely(mp && mp->mp_subscript))
#endif
    {
        PyObject* result;
        PyObject *py_slice, *py_start, *py_stop;
        if (_py_slice) {
            py_slice = *_py_slice;
        } else {
            PyObject* owned_start = NULL;
            PyObject* owned_stop = NULL;
            if (_py_start) {
                py_start = *_py_start;
            } else {
                if (has_cstart) {
                    owned_start = py_start = PyInt_FromSsize_t(cstart);
                    if (unlikely(!py_start)) goto bad;
                } else
                    py_start = Py_None;
            }
            if (_py_stop) {
                py_stop = *_py_stop;
            } else {
                if (has_cstop) {
                    owned_stop = py_stop = PyInt_FromSsize_t(cstop);
                    if (unlikely(!py_stop)) {
                        Py_XDECREF(owned_start);
                        goto bad;
                    }
                } else
                    py_stop = Py_None;
            }
            py_slice = PySlice_New(py_start, py_stop, Py_None);
            Py_XDECREF(owned_start);
            Py_XDECREF(owned_stop);
            if (unlikely(!py_slice)) goto bad;
        }
#if CYTHON_USE_TYPE_SLOTS
        result = mp->mp_subscript(obj, py_slice);
#else
        result = PyObject_GetItem(obj, py_slice);
#endif
        if (!_py_slice) {
            Py_DECREF(py_slice);
        }
        return result;
    }
    PyErr_Format(PyExc_TypeError,
        "'%.200s' object is unsliceable", Py_TYPE(obj)->tp_name);
bad:
    return NULL;
}

/* GetException */
      #if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb) {
#endif
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

/* Import */
        static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_VERSION_HEX < 0x03030000
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.')) {
                #if PY_VERSION_HEX < 0x03030000
                PyObject *py_level = PyInt_FromLong(1);
                if (!py_level)
                    goto bad;
                module = PyObject_CallFunctionObjArgs(py_import,
                    name, global_dict, empty_dict, list, py_level, NULL);
                Py_DECREF(py_level);
                #else
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                #endif
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_VERSION_HEX < 0x03030000
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_VERSION_HEX < 0x03030000
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* ImportFrom */
        static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* GetModuleGlobalName */
        static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name) {
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
    result = PyDict_GetItem(__pyx_d, name);
    if (likely(result)) {
        Py_INCREF(result);
    } else {
#else
    result = PyObject_GetItem(__pyx_d, name);
    if (!result) {
        PyErr_Clear();
#endif
        result = __Pyx_GetBuiltinName(name);
    }
    return result;
}

/* CodeObjectCache */
          static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, (size_t)new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
          #include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    py_code = __pyx_find_code_object(c_line ? c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        PyThreadState_GET(), /*PyThreadState *tstate,*/
        py_code,             /*PyCodeObject *code,*/
        __pyx_d,      /*PyObject *globals,*/
        0                    /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntToPy */
          static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* PyUCS4InUnicode */
          static CYTHON_INLINE int __Pyx_UnicodeContainsUCS4(PyObject* unicode, Py_UCS4 character) {
#if CYTHON_PEP393_ENABLED
    const int kind = PyUnicode_KIND(unicode);
    if (likely(kind != PyUnicode_WCHAR_KIND)) {
        Py_ssize_t i;
        const void* udata = PyUnicode_DATA(unicode);
        const Py_ssize_t length = PyUnicode_GET_LENGTH(unicode);
        for (i=0; i < length; i++) {
            if (unlikely(character == PyUnicode_READ(kind, udata, i))) return 1;
        }
        return 0;
    }
#endif
    return __Pyx_PyUnicodeBufferContainsUCS4(
        PyUnicode_AS_UNICODE(unicode),
        PyUnicode_GET_SIZE(unicode),
        character);
}
static CYTHON_INLINE int __Pyx_PyUnicodeBufferContainsUCS4(Py_UNICODE* buffer, Py_ssize_t length, Py_UCS4 character) {
    Py_UNICODE uchar;
    Py_UNICODE* pos;
    #if Py_UNICODE_SIZE == 2
    if (character > 65535) {
        Py_UNICODE high_val, low_val;
        high_val = (Py_UNICODE) (0xD800 | (((character - 0x10000) >> 10) & ((1<<10)-1)));
        low_val  = (Py_UNICODE) (0xDC00 | ( (character - 0x10000)        & ((1<<10)-1)));
        for (pos=buffer; pos < buffer+length-1; pos++) {
            if (unlikely(high_val == pos[0]) & unlikely(low_val == pos[1])) return 1;
        }
        return 0;
    }
    #endif
    uchar = (Py_UNICODE) character;
    for (pos=buffer; pos < buffer+length; pos++) {
        if (unlikely(uchar == pos[0])) return 1;
    }
    return 0;
}

/* CIntFromPyVerify */
          #define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntToPy */
          static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
          static CYTHON_INLINE uint8_t __Pyx_PyInt_As_uint8_t(PyObject *x) {
    const uint8_t neg_one = (uint8_t) -1, const_zero = (uint8_t) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(uint8_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(uint8_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (uint8_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint8_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(uint8_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(uint8_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8_t) >= 2 * PyLong_SHIFT) {
                            return (uint8_t) (((((uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint8_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8_t) >= 3 * PyLong_SHIFT) {
                            return (uint8_t) (((((((uint8_t)digits[2]) << PyLong_SHIFT) | (uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint8_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8_t) >= 4 * PyLong_SHIFT) {
                            return (uint8_t) (((((((((uint8_t)digits[3]) << PyLong_SHIFT) | (uint8_t)digits[2]) << PyLong_SHIFT) | (uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (uint8_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(uint8_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint8_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(uint8_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint8_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (uint8_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(uint8_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(uint8_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(uint8_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8_t) - 1 > 2 * PyLong_SHIFT) {
                            return (uint8_t) (((uint8_t)-1)*(((((uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(uint8_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8_t) - 1 > 2 * PyLong_SHIFT) {
                            return (uint8_t) ((((((uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(uint8_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8_t) - 1 > 3 * PyLong_SHIFT) {
                            return (uint8_t) (((uint8_t)-1)*(((((((uint8_t)digits[2]) << PyLong_SHIFT) | (uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(uint8_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8_t) - 1 > 3 * PyLong_SHIFT) {
                            return (uint8_t) ((((((((uint8_t)digits[2]) << PyLong_SHIFT) | (uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(uint8_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8_t) - 1 > 4 * PyLong_SHIFT) {
                            return (uint8_t) (((uint8_t)-1)*(((((((((uint8_t)digits[3]) << PyLong_SHIFT) | (uint8_t)digits[2]) << PyLong_SHIFT) | (uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(uint8_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(uint8_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(uint8_t) - 1 > 4 * PyLong_SHIFT) {
                            return (uint8_t) ((((((((((uint8_t)digits[3]) << PyLong_SHIFT) | (uint8_t)digits[2]) << PyLong_SHIFT) | (uint8_t)digits[1]) << PyLong_SHIFT) | (uint8_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(uint8_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint8_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(uint8_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(uint8_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            uint8_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (uint8_t) -1;
        }
    } else {
        uint8_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (uint8_t) -1;
        val = __Pyx_PyInt_As_uint8_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to uint8_t");
    return (uint8_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to uint8_t");
    return (uint8_t) -1;
}

/* CIntFromPy */
          static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntFromPy */
          static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CheckBinaryVersion */
          static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* InitStrings */
          static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
static CYTHON_INLINE char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if CYTHON_COMPILING_IN_CPYTHON && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
#if PY_VERSION_HEX < 0x03030000
        char* defenc_c;
        PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
        if (!defenc) return NULL;
        defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        {
            char* end = defenc_c + PyBytes_GET_SIZE(defenc);
            char* c;
            for (c = defenc_c; c < end; c++) {
                if ((unsigned char) (*c) >= 128) {
                    PyUnicode_AsASCIIString(o);
                    return NULL;
                }
            }
        }
#endif
        *length = PyBytes_GET_SIZE(defenc);
        return defenc_c;
#else
        if (__Pyx_PyUnicode_READY(o) == -1) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        if (PyUnicode_IS_ASCII(o)) {
            *length = PyUnicode_GET_LENGTH(o);
            return PyUnicode_AsUTF8(o);
        } else {
            PyUnicode_AsASCIIString(o);
            return NULL;
        }
#else
        return PyUnicode_AsUTF8AndSize(o, length);
#endif
#endif
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (PyInt_Check(x) || PyLong_Check(x))
#else
  if (PyLong_Check(x))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = PyNumber_Long(x);
  }
  #else
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Long(x);
  }
  #endif
#else
  res = PyNumber_Int(x);
#endif
  if (res) {
#if PY_MAJOR_VERSION < 3
    if (!PyInt_Check(res) && !PyLong_Check(res)) {
#else
    if (!PyLong_Check(res)) {
#endif
      PyErr_Format(PyExc_TypeError,
                   "__%.4s__ returned non-%.4s (type %.200s)",
                   name, name, Py_TYPE(res)->tp_name);
      Py_DECREF(res);
      return NULL;
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(x);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
